#include <linux/device.h>
#include <linux/fs.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/moduleparam.h>
#include <linux/list.h>
#include <linux/cdev.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/syscalls.h>
#include <asm/cacheflush.h>
#include <linux/mman.h>
#include <linux/delay.h>
#include <asm/io.h>

#define CMEM_MAP_DSPMEM	0x1
#define CMEM_MAP_ARMMEM	0x2
#define CMEM_MALLOC_MEM 0x3
#define CMEM_FREE_MEM	0x4
#define CMEM_SND_INTR	0x5
#define CMEM_SET_DBGMEM	0x6
#define CMEM_STOP_CPU	0x7
#define CMEM_LOAD_CPU	0x8
#define CMEM_CACHE_INVALIDATE 0x9
#define CMEM_CACHE_CLEAN 0xa
#define CMEM_CACHE_CLEAN_INVALIDATE 0xb
#define CMEM_DSP_STARTMEM 0xc
//#define DSP_MEM_ADDR		0x82000000
//#define DSP_MEM_SIZE		(32*1024*1024)

typedef struct {
	u_long phyaddr;		/* tell by user */
	u_long virtaddr;		/* return to user */
	u_long memsize;		/* tell by user and will modify by kernel for align */
} DSP_MEM_INFO;

typedef struct {
	u_int addr;/* user virt addr */
	u_int size;/* sync size */
} CACHE_SYNC_INFO;

extern void cpu_stop(int procId);
extern void cpu_load(int procId,void *file);
extern void ipc_setup(void);
extern void ipc_close(void);
extern char g_cpufile[3][64];

static void *g_pdspmem = NULL;
static DSP_MEM_INFO g_meminfo;

#define ALIGN_PAGE_SIZE(size)		((size + PAGE_SIZE - 1)&(~(PAGE_SIZE - 1)))  /* align upper */
void* cmem_malloc_reserverd(int size) 
{
    int order;
    char *buf;
    unsigned long addr;

    /* align with PAGE_SIZE, the upper side */
    size = ALIGN_PAGE_SIZE(size);
    order = get_order(size);
	
    buf = (char *)__get_free_pages(GFP_KERNEL|GFP_DMA,order);   
    addr = (unsigned long)buf;
    if(buf) 
    {
        while(size > 0) 
       	{    
            SetPageReserved(virt_to_page(addr));
            addr += PAGE_SIZE;
            size -= PAGE_SIZE;
        }
    }
	else
	{
		printk("[CMEM]: cannot alloc mem(%d) form kenerl region\n",size);
    }
	return buf;
}

void cmem_free_reserverd(unsigned long addr, int size)
{
	unsigned int temp_size, temp_addr;

	if (!addr)
		return;
	
	temp_addr = addr;	
	temp_size = PAGE_SIZE << (get_order(size));
	while(temp_size > 0) 
	{
		ClearPageReserved(virt_to_page(temp_addr));
		temp_addr += PAGE_SIZE;
		temp_size -= PAGE_SIZE;
	}

	free_pages(addr, get_order(size));
}

static int g_use_cache = 0;
static int cmem_mmap(struct file *file, struct vm_area_struct *vma)
{
	//printk("----------------------------------------->vma->vm_pgoff = 0x%x,phy=0x%x\n", vma->vm_pgoff,g_meminfo.phyaddr);
	if( (vma->vm_pgoff == (g_meminfo.phyaddr>>PAGE_SHIFT))
		&& ((vma->vm_end-vma->vm_start) == g_meminfo.memsize) ) 
	{
		if(g_use_cache==0) {
			//vma->vm_flags = 0x100073;
			//vma->vm_page_prot = 0x10f;
			vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
		} else {
			//vma->vm_flags |= VM_IO;
			//vma->vm_flags |= VM_RESERVED;
			vma->vm_page_prot = 0x183|L_PTE_MT_WRITEBACK;
		}
		//printk("<duhao>1_flags=%lx,prot=%lx\n",vma->vm_flags,vma->vm_page_prot);
		if( remap_pfn_range(vma,vma->vm_start,vma->vm_pgoff,g_meminfo.memsize,vma->vm_page_prot) ) {
			printk("mmap dsp mem to user space error\n");
			return -EAGAIN;
		}
		//printk("<duhao>2_flags=%lx,prot=%lx\n",vma->vm_flags,vma->vm_page_prot);
	}
	else
		return -EINVAL;
	return 0;
}

static int g_cache_line_sz = 0;

struct semaphore cmem_sem;
static int cmem_ioctl(struct file *file,unsigned int cmd, unsigned long arg)
{
	down(&cmem_sem);
	switch(cmd)
	{
		case CMEM_MAP_DSPMEM:
		{
			u_long virtaddr;
			if(arg == 0)
			{
				up(&cmem_sem);
				return -EINVAL;
			}

			//printk("<duhao>----------->ioctl cmem_map_dspmem\n");
			
			copy_from_user((void *)&g_meminfo,(void *)arg,sizeof(g_meminfo));
			g_use_cache = (g_meminfo.memsize & 0x80000000) ? 1 : 0;
			g_meminfo.memsize &= 0x7fffffff;
			if( g_meminfo.memsize == 0 )
			{
				up(&cmem_sem);
				return -EINVAL;
			}
			g_meminfo.memsize = (g_meminfo.memsize/PAGE_SIZE)*PAGE_SIZE;
			g_meminfo.phyaddr = ((g_meminfo.phyaddr+PAGE_SIZE-1)/PAGE_SIZE)*PAGE_SIZE;
			g_meminfo.virtaddr = 0;
			
			if(g_pdspmem == NULL) {
				if(g_use_cache) {
					g_pdspmem = ioremap_cached(g_meminfo.phyaddr,g_meminfo.memsize);
				} else {
					g_pdspmem = ioremap_nocache(g_meminfo.phyaddr,g_meminfo.memsize);
				}
				//g_pdspmem = 0xd0000000;
			}
		
			if(g_pdspmem==NULL) {
				printk("cmem ioremap dsp mem error\n");
				up(&cmem_sem);
				return -ENOMEM;
			}
			
			/*
			MAP_SHARED		ָеҳԱ̹ 
			MAP_PRIVATE		ָеҳܱ̹ 
			MAP_ANONYMOUS	ʾģκļ޹
			MAP_FIXED		ʼַɲaddrָ
			MAP_NORESERVE	ԤȼҳĿ
			MAP_LOCKED		ӳסʾ򲻻ᱻûswap
			*/
			down_write(&current->mm->mmap_sem);
			virtaddr = do_mmap_pgoff(file,g_pdspmem,g_meminfo.memsize,PROT_READ|PROT_WRITE,MAP_SHARED,g_meminfo.phyaddr>>PAGE_SHIFT);
			//virtaddr = do_mmap_pgoff(file,0,g_meminfo.memsize,PROT_READ|PROT_WRITE,MAP_PRIVATE|MAP_LOCKED,g_meminfo.phyaddr>>PAGE_SHIFT);
			up_write(&current->mm->mmap_sem);

			if(virtaddr<0)/* do_mmap_pgoff error */
			{
				up(&cmem_sem);
				return virtaddr;
			}
			g_meminfo.virtaddr = virtaddr;

			//printk("<duhao>----------->ioctl cmem_map_dspmem(%x),virt=%x,phys=%x,size=%d\n",
			//	g_pdspmem,g_meminfo.virtaddr,g_meminfo.phyaddr,g_meminfo.memsize);
			copy_to_user((void *)arg,(void *)&g_meminfo,sizeof(g_meminfo));

			break;
		}
		case CMEM_MAP_ARMMEM:
		{
			if(g_pdspmem == NULL)
			{
				printk("you must call CMEM_MAP_DSPMEM first\n");
				up(&cmem_sem);
				return -ENOMEM;
			}
			copy_to_user((void *)arg,(void *)&g_pdspmem,sizeof(u_long));
			break;
		}
		case CMEM_MALLOC_MEM:	//wangqian add for 
		{
			u_int virtaddr;
			void *mem;

			copy_from_user((void *)&g_meminfo,(void *)arg,sizeof(g_meminfo));
			g_use_cache = (g_meminfo.memsize & 0x80000000) ? 1 : 0;
			g_meminfo.memsize &= 0x7fffffff;

			if( g_meminfo.memsize == 0 )
			{
				up(&cmem_sem);
				return -EINVAL;	
			}
			if(g_meminfo.memsize & (g_meminfo.memsize - 1)) //check align with 2^n
			{
				up(&cmem_sem);
				return -EINVAL;
			}
			mem = (void *)cmem_malloc_reserverd(g_meminfo.memsize);
			g_meminfo.phyaddr = virt_to_phys((void *)mem);

			//mmap
			down_write(&current->mm->mmap_sem);
			virtaddr = do_mmap_pgoff(file,0,g_meminfo.memsize,0x3,0x1,g_meminfo.phyaddr>>PAGE_SHIFT);
			up_write(&current->mm->mmap_sem);

			if(virtaddr<0)/* do_mmap_pgoff error */
			{
				up(&cmem_sem);
				return virtaddr;
			}
			g_meminfo.virtaddr = virtaddr;

			printk("cmem: [MAP]size=0x%lx,phy=0x%lx,virt=0x%lx\n",
				g_meminfo.memsize, g_meminfo.phyaddr,g_meminfo.virtaddr);
			copy_to_user((void *)arg,(void *)&g_meminfo,sizeof(g_meminfo));

			break;
		}
		case CMEM_FREE_MEM:
		{
			u_int virtaddr;
			copy_from_user((void *)&g_meminfo,(void *)arg,sizeof(g_meminfo));
			g_meminfo.memsize &= 0x7fffffff;

			if( g_meminfo.memsize == 0 )
			{
				up(&cmem_sem);
				return -EINVAL;	
			}
			if(g_meminfo.memsize & (g_meminfo.memsize - 1)) //check align with 2^n
			{
				up(&cmem_sem);
				return -EINVAL;		
			}				

			printk("cmem: [FREE]size=0x%lx,phy=0x%lx,virt=0x%lx\n",
				g_meminfo.memsize, g_meminfo.phyaddr,g_meminfo.virtaddr);
			//munmapͷ
			do_munmap(current->mm, g_meminfo.virtaddr, g_meminfo.memsize);

			virtaddr = (u_int)phys_to_virt(g_meminfo.phyaddr);
			cmem_free_reserverd(virtaddr, g_meminfo.memsize);

			break;
		}
		case CMEM_SND_INTR:
		{
			extern int ipc_send_int(int procId, u_int value);
			int procId = (int)arg;
			if(ipc_send_int(procId,1) == -1)
			{
				up(&cmem_sem);
				return -1;
			}
			break;
		}
		case CMEM_SET_DBGMEM:
		{
			extern int cmem_dbgmem_init(u_int phy_dbgmem);
			if(cmem_dbgmem_init(arg) < 0)
			{
				up(&cmem_sem);
				return -1;
			}
			break;
		}
		case CMEM_STOP_CPU:
		{
			int procId = (int)arg;
			cpu_stop(procId);
			break;
		}
		case CMEM_LOAD_CPU:
		{
			int procId = (int)arg;
			cpu_load(procId,NULL);
			break;			
		}
		case CMEM_CACHE_INVALIDATE: 
		{
			//extern struct cpu_cache_fns cpu_cache;
			CACHE_SYNC_INFO csi;
			copy_from_user((void *)&csi,(void *)arg,sizeof(csi));
			//cpu_cache.coherent_user_range(csi.addr, csi.addr + csi.size);
			//cpu_cache.coherent_kern_range(g_pdspmem, g_pdspmem + csi.size);
			//clean_dcache_area(csi.addr, csi.size);
			{
				u_int start,end,cache_line_sz;
				cache_line_sz = g_cache_line_sz;
				start = csi.addr;
				end = start + csi.size;
				if(end % cache_line_sz) {
					end += cache_line_sz;
				}
				start &= ~(cache_line_sz - 1);
				while (start < end) {
					/* Invalidate DCache single entry (MVA) */
					asm volatile ("mcr p15, 0, %0, c7, c6, 1" : : "r" (start));
					start += cache_line_sz;/* cache line size */
				}
			}
			break;
		}
        case CMEM_CACHE_CLEAN: 
        {
            CACHE_SYNC_INFO csi;
            copy_from_user((void *)&csi,(void *)arg,sizeof(csi));
            {
                u_int start,end,cache_line_sz;
                cache_line_sz = g_cache_line_sz;
                start = csi.addr;
                end = start + csi.size;
                if(end % cache_line_sz)
                {
                    end += cache_line_sz;
                }
                start &= ~(cache_line_sz - 1);
                while (start < end) 
                {
                    /* Clean DCache single entry (MVA) */
                    asm volatile ("mcr p15, 0, %0, c7, c10, 1" : : "r" (start));
                    start += cache_line_sz;/* cache line size */
                }
            }
            break;
        }
        case CMEM_CACHE_CLEAN_INVALIDATE:
        {
            CACHE_SYNC_INFO csi;
            copy_from_user((void *)&csi,(void *)arg,sizeof(csi));
            {
                u_int start,end,cache_line_sz;
                cache_line_sz = g_cache_line_sz;
                start = csi.addr;
                end = start + csi.size;
                if(end % cache_line_sz)
                {
                    end += cache_line_sz;
                }
                start &= ~(cache_line_sz - 1);
                while (start < end)
                {
                    /* Clean and Invalidate DCache single entry (MVA) */
                    asm volatile ("mcr p15, 0, %0, c7, c14, 1" : : "r" (start));
                    start += cache_line_sz;/* cache line size */
                }
            }
            break;
        }
		default:
			up(&cmem_sem);
			return -EINVAL;
	}
	
	up(&cmem_sem);
	return 0;
}

static int cmem_open(struct inode *inode, struct file *filep)
{
	u_int cache_info;
	asm volatile("mrc p15, 0, %0, c0, c0, 1" :"=r"(cache_info)::"memory");
	g_cache_line_sz = ( 1 << ( ((cache_info>>16) & 0xf) ) ) * sizeof(u_int);
	printk("arm cache line size is 0x%x\n",g_cache_line_sz);
	return 0;
}

static int cmem_release(struct inode *inode, struct file *filep)
{
	return 0;
}

static struct file_operations cmem_fops = {
	.owner = THIS_MODULE,
	.open = cmem_open,
	.release = cmem_release,
	.unlocked_ioctl = cmem_ioctl,
	.mmap = cmem_mmap
};


static ssize_t cmem_write(struct file *filp,const char __user *buf,unsigned long len,void *data)
{
	char *p = NULL;
	u_char input[256];
	int cmd;
	int procId = 2;

	char name[64];
	strcpy(name,filp->f_dentry->d_name.name);

	if(strcmp(name,"vid")==0) {
		procId = 0;
	} else if(strcmp(name,"vps")==0) {
		procId = 1;
	} else {/* dsp */
		procId = 2;
	}

	if (len >= 256) {
		return -ENOMEM;	
	}
	if (copy_from_user(input,buf,len )) {
		return -EFAULT;
	}

	if(input[len-1]=='\n') {
		input[len-1]=0;
	} else {
		input[len]=0;
	}
	
	cmd = simple_strtoul(input,NULL,16);
	if(cmd==0) { /* stop cpu */
		cpu_stop(procId);
	} else if(cmd==1) {/* load cpu */
		p = strchr(input,' ');
		if(p) {
			strcpy(g_cpufile[procId],p+1);
			printk("set core%d load filename to [%s]\n",procId,p+1);
		}
		cpu_load(procId,NULL);
	}
	return len;
}

static int cmem_init(void)
{
	struct proc_dir_entry *pentry;
	printk("cmem1\n");
	ipc_setup();
	init_MUTEX(&cmem_sem);

    if (register_chrdev(232,"cmem",&cmem_fops) < 0)
    {
    	printk("register cmem driver failed.\n");
		return -ENODEV;
    }
	memset((void*)&g_meminfo,0,sizeof(g_meminfo));
	printk("register cmem driver successed.\n");


	if(!proc_mkdir("cmem",NULL))
		return -ENOMEM;
	
	
	pentry = create_proc_entry("cmem/vid",0,NULL);
	pentry->write_proc = cmem_write;

	pentry = create_proc_entry("cmem/vps",0,NULL);
	pentry->write_proc = cmem_write;

	pentry = create_proc_entry("cmem/dsp",0,NULL);
	pentry->write_proc = cmem_write;
	
	return 0;
}
static void cmem_cleanup(void)
{
	remove_proc_entry("cmem/dsp",NULL);
	remove_proc_entry("cmem/vps",NULL);
	remove_proc_entry("cmem/vid",NULL);
	remove_proc_entry("cmem",NULL);

	if(g_pdspmem)
		iounmap((void*)g_pdspmem);
	unregister_chrdev(232,"cmem");

	ipc_close();
}

module_init(cmem_init);
module_exit(cmem_cleanup);
MODULE_LICENSE("GPL");






















