#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/io.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include "cmem.h"


delay()
{
	int del_cnt;
	for(del_cnt=0;del_cnt<200;del_cnt++);
}

typedef int (*isr_cb) (void *args);

typedef struct _ipc_obj_ {
    int asserted;
    isr_cb fxn;
    void *args;
} ipc_obj;

static ipc_obj g_ipc_objs[DM740_NUMPROCS];
static char *g_dbgmem[DM740_NUMPROCS];
static char *g_virt_dbgmem = NULL;
u_int g_phys_dbgmem = 0;

static void ipc_enable_int(int procId)
{
    if (procId == CORE_C674) {
		or_io(MAILBOX_IRQENABLE_SET,1<<(MAILBOX_NUMBER_0*2));
    } else if (procId == CORE_VID) {
		or_io(MAILBOX_IRQENABLE_SET,1<<(MAILBOX_NUMBER_6*2));
    } else if (procId == CORE_VPS) {
		or_io(MAILBOX_IRQENABLE_SET,1<<(MAILBOX_NUMBER_8*2));    
	}
}

static void ipc_disable_int(int procId)
{
    if (procId == CORE_C674) {
		or_io(MAILBOX_IRQENABLE_CLR,1<<(MAILBOX_NUMBER_0*2));
    } else if (procId == CORE_VID) {
		or_io(MAILBOX_IRQENABLE_CLR,1<<(MAILBOX_NUMBER_6*2));
    } else if (procId == CORE_VPS) {
		or_io(MAILBOX_IRQENABLE_CLR,1<<(MAILBOX_NUMBER_8*2));    
	}
}

static int ipc_wait_int_clr (int procId)
{
    if (procId == CORE_C674) {
		while( get_io(MAILBOX_MESSAGE_3) & 0x3f );
    } else if (procId == CORE_VID) {
		while( get_io(MAILBOX_MESSAGE_4) & 0x3f );
    } else if (procId == CORE_VPS) {
		while( get_io(MAILBOX_MESSAGE_5) & 0x3f );
	}

    return 0 ;
}


static u_int ipc_clr_int(int procId)
{
    u_int regval = 0;

    if (procId == CORE_C674) {
		/* get the entry from the mailbox FIFO */
		regval = get_io(MAILBOX_MESSAGE_0);
		/* Clear the IRQ status, If more in the mailbox FIFO, it will re-assert. */
        or_io(MAILBOX_IRQSTATUS_CLEAR,1<<(MAILBOX_NUMBER_0*2));
    } else if (procId == CORE_VID) {
		/* get the entry from the mailbox FIFO */
		regval = get_io(MAILBOX_MESSAGE_6);
        /* Clear the IRQ status, If more in the mailbox FIFO, it will re-assert. */
		or_io(MAILBOX_IRQSTATUS_CLEAR,1<<(MAILBOX_NUMBER_6*2));
    } else if (procId == CORE_VPS) {
		regval = get_io(MAILBOX_MESSAGE_8);/* get the entry from the mailbox FIFO */
        /* Clear the IRQ status, If more in the mailbox FIFO, it will re-assert. */
		or_io(MAILBOX_IRQSTATUS_CLEAR,1<<(MAILBOX_NUMBER_8*2));
    }

    return regval;
}

static int ipc_check_clr_int(void)
{
    u_int procId;

    if( get_io(MAILBOX_MSGSTATUS_0) != 0 ) {
		procId = CORE_C674;
		ipc_clr_int(procId);
		g_ipc_objs[procId].asserted = 1;
    }
    if( get_io(MAILBOX_MSGSTATUS_6) != 0 ) {
		procId = CORE_VID;
		ipc_clr_int(procId);
		g_ipc_objs[procId].asserted = 1;
    }
    if( get_io(MAILBOX_MSGSTATUS_8) != 0 ) {
		procId = CORE_VPS;
		ipc_clr_int(procId);
		g_ipc_objs[procId].asserted = 1;
    }
    return 0;
}

static irqreturn_t ipc_isr(int irq, void *dev)
{
    int i;
	
	ipc_check_clr_int();
	
    for (i = 0 ; i < DM740_NUMPROCS ; i++) {
		ipc_obj *pobj = &g_ipc_objs[i];
        if (pobj->asserted == 1) {
            pobj->asserted = 0;
            if(pobj->fxn != NULL) {
				pobj->fxn(pobj->args);
            }
		}
    }
    return 0;
}


int ipc_register_int(int procId,isr_cb fxn,void *args)
{
	g_ipc_objs[procId].fxn  = fxn;
	g_ipc_objs[procId].args = args;
	ipc_enable_int(procId);/* Enable hardware interrupt. */

    return 0;
}


static int dbg_msg(void *args)
{
	int id = (int)args;
	int i,count1,count2;
	volatile u_int r,w;

	u_char *dbg_base = NULL;
	if(id==CORE_C674) {
		dbg_base = g_dbgmem[2];
	} else if(id==CORE_VID) {
		dbg_base = g_dbgmem[0];
	} else if(id==CORE_VPS) {
		dbg_base = g_dbgmem[1];
	}

	if(dbg_base==0)
		return -1;
	
	w = *(volatile u_int *)(dbg_base);
	r = *(volatile u_int *)(dbg_base + 4);

	//printk("<dbg>r=%d,w=%d\n",r,w);

	if(r == w) {
		//printk("<0>[core%d]r(%d)=w(%d)\n",id,r,w);
		return 0;
	}


#if 1
	if(r<w) {
		count1 = w - r;
		count2 = 0;
	} else {/* r>w */
		count1 = (OCMC_MEM_BUF_LEN - 1) - r;
		count2 = w;
	}
	
	if( (count1 + count2) >= (OCMC_MEM_BUF_LEN - 1) ) {
		printk("netra ipc lost messg.\n");
		*(volatile u_int *)(dbg_base + 4) = w;
		return -2;/* drop all msg */
	}

	for(i=0;i<count1;i++) {
		if( *(u_char *)((u_int)dbg_base + ((r+1+i)<<8) ) ) {
			//printk("<0>[core%d-r%d-w%d-i%d]%s", id, r, w, i, dbg_base + (r+1+i)*256);
			printk("<0>[core%d]%s", id, (u_char *)((u_int)dbg_base + ((r+1+i)<<8)) );
			//printk((u_char *)((u_int)dbg_base + ((r+1+i)<<8)));
		}
	}

	for(i=0;i<count2;i++) {
		if( *(u_char *)((u_int)dbg_base + ((i+1)<<8)) ) {
			//printk("<0>[core%d-r%d-w%d-i%d]%s", id, r, w, i, dbg_base + (i+1)*256);
			printk("<0>[core%d]%s", id, (u_char *)((u_int)dbg_base + ((i+1)<<8) ) );
			//printk((u_char *)((u_int)dbg_base + ((i+1)<<8) ));
		}
	}
	
	*(volatile u_int *)(dbg_base + 4) = w;
#else
	{
		u_int npos = ((r + 1)==256) ? 0 : r + 1;
		if( *(dbg_base + npos*256) )
			printk("[core%d-r%d-w%d]%s", id, r, w, dbg_base + npos*256);
		*(volatile u_int *)(dbg_base + 4) = npos;
	}
#endif
	return 0;
}


/*--------------------------------------------------------------------------------------------------------------*/

static void hdvpss_clk_enable()
{
	set_io(RM_HDVPSS_RSTCTRL,0);
    printk("\tPower Enabled.....  \n");

    /* Enabling DSS Clocks */
	set_io(CM_HDVPSS_CLKSTCTRL,2);
	delay();
	set_io(CM_HDVPSS_HDVPSS_CLK_CTRL,2);
	delay();
	/* Enable HDMI Clocks */
	set_io(CM_HDVPSS_HDMI_CLKCTRL,2);
	delay();

	printk("\tPRCM for DSS in Progress, Please wait.....  \n");
	while ( (get_io(CM_HDVPSS_CLKSTCTRL) & 0x100) != 0x100)
	{
	}

	printk("\tPRCM for DSS ModuleMode in Progress, Please wait.....  \n");
	while ( (get_io(CM_HDVPSS_HDVPSS_CLK_CTRL) & 0x2) != 0x2)
	{
	}

    /*Deasserting resets */
    set_io(RM_HDVPSS_RSTCTRL,0);
	printk("\tPRCM for DSS Done Successfully.....  \n");
}

static void hdmi_clk_enable(void)
{
   /* Enable HDMI Clocks */
	set_io(CM_HDVPSS_HDMI_CLKCTRL,2);
	delay();
}

static void i2c_clk_enable(void)
{	
    printk("PRCM for I2c is in Progress......");
    //set_io(CM_ALWON_I2C_02_CLKCTRL) = 2; /* Enable Power Domain Transition for I2C0 */
    set_io(CM_ALWON_I2C_13_CLKCTRL, 2); /* Enable Power Domain Transition for I2C1 */
    
    /* Check to see module is functional */
    //while((( get_io(CM_ALWON_I2C_02_CLKCTRL) & 0x70000) >> 16) != 0) ;
    while((( get_io(CM_ALWON_I2C_13_CLKCTRL) & 0x70000) >> 16) != 0) ;
    printk("done.\n");
}
 
static void MailboxEnable(void)
{
    printk("\tPRCM for mailbox is in Progress  \n");
      
    set_io(CM_ALWON_L3_SLOW_CLKSTCTRL,       2);
    set_io(CM_ALWON_MAILBOX_CLKCTRL,        2); /*Enable Power Domain Transition for mailbox */


    printk("\tmailbox Clocks are active \n");

    /* Check to see module is functional */
    while(((get_io(CM_ALWON_MAILBOX_CLKCTRL) & 0x30000) >> 16) != 0) ;

    printk("\tPRCM for mailbox is Done Successfully  \n");
}


static void video_pll(u_int VIDEO_N, u_int VIDEO_P, 
	u_int VIDEO_INTFREQ1, u_int VIDEO_FRACFREQ1, u_int VIDEO_MDIV1, 
	u_int VIDEO_INTFREQ2, u_int VIDEO_FRACFREQ2, u_int VIDEO_MDIV2, 
	u_int VIDEO_INTFREQ3, u_int VIDEO_FRACFREQ3, u_int VIDEO_MDIV3, 
	u_int B1, u_int C2)
{
	
}


static void video_drr_pll(void)
{
	printk("Netra Video PLL Init is in Progress, Please wait ..... \n");
    video_pll(0x6E, 0x2, 0xB, 0x0, 0x5, 0xA, 0x0, 0x2, 0xA, 0x0, 0x2, 0x7, 0x7);
    printk("Netra Video PLL Init is Done..... \n");
}

static void ducati_clk_enable(void)
{
	int i;
	u_int ducati_iovirt = 0;
	printk("PRCM for DucatiSS is in Progress \n");

	set_io(RM_DEFAULT_RSTCTRL,   0xEF); /*Enable the Ducati Logic*/
	set_io(CM_DEFAULT_DUCATI_CLKSTCTRL, 2); /*Enable Power Domain Transition*/
	set_io(CM_DEFAULT_DUCATI_CLKCTRL,   2); /*Enable Ducati Clocks*/


	/*Check CLKIN200TR & CLKINTR  are active*/
	while(((get_io(CM_DEFAULT_DUCATI_CLKSTCTRL)&0x300)>>8)!=3);
         printk("Clock is Active  \n");

   
	printk("Tests Executed: %x \n",get_io(RM_DEFAULT_RSTCTRL));
	printk("Tests Executed: %x \n",get_io(RM_DEFAULT_RSTST));
	printk("DUCATI_BASE_ADDR:\n");

	ducati_iovirt = (u_int)ioremap(DUCATI_BASE_ADDR,0x100);
	printk("DUCATI_BASE_ADDR virt =0x%08x\n",ducati_iovirt);
	
	/*Write Ducate IRAM Boot Image */
	printk("DUCATI_BASE_ADDR:%x\n",*(volatile u_int *)(ducati_iovirt));

	*(volatile u_int *)(ducati_iovirt) = 0x10000;
	*(volatile u_int *)(ducati_iovirt + 0x4) = 0x9;
	*(volatile u_int *)(ducati_iovirt + 0x8) = 0xE7FEE7FE;

    set_io(RM_DEFAULT_RSTCTRL,   0xE3); //0xE3

	/*Check for Ducati M3_0 & M3_1 out of Reset*/
	while(((get_io(RM_DEFAULT_RSTST)&0x1C))!=0x1C) ;

	/*Check Module is in Functional Mode */
	while(((get_io(CM_DEFAULT_DUCATI_CLKCTRL)&0x30000)>>16)!=0) ;

    printk("\tPRCM for DucatiSS is Done Successfully  \n","Output",1,1,1);
}

void timer_clk_enable(void)
{
	set_io(CM_TIMER4_CLKSEL, 0x2);/* Selects CLKIN (20MHz) */
	set_io(CM_ALWON_TIMER_4_CLKCTRL, 0x2);/* TIMER 4 enable for dsp bios */
	while( ( ( get_io(CM_ALWON_TIMER_4_CLKCTRL) & 0x30000 ) >> 16 ) != 0 );	
}

/*--------------------------------------------------------------------------------------------------------------*/

void ipc_setup(void)
{
    int i;
	
	ducati_clk_enable();
printk("ipc1\n");
	i2c_clk_enable();
printk("ipc2\n");
	MailboxEnable();
printk("ipc3\n");
	hdvpss_clk_enable();
printk("ipc4\n");
	hdmi_clk_enable();
printk("ipc5\n");
	video_drr_pll();
printk("ipc6\n");
	timer_clk_enable();
	
   // set_io(CM_ALWON_MAILBOX_CLKCTRL, 2); /* Enable Power Domain Transition for mailbox */
    /* Check to see module is functional */
   // while(((get_io(CM_ALWON_MAILBOX_CLKCTRL) & 0x70000) >> 16) != 0) ;

	for (i = 0; i < DM740_NUMPROCS; i++ ) {
		g_ipc_objs[i].asserted = 0;
		ipc_register_int(i, dbg_msg, (void *)i);
	}
	i = request_irq(HOSTINT, ipc_isr, 0, "SYSLINK", NULL);

	printk("register ipc irq ret is %d\n",i);
}

void ipc_close(void)
{
	free_irq(HOSTINT,NULL);
	iounmap(g_virt_dbgmem);

}

int ipc_send_int(int procId, u_int value)
{
	//printk("ipc_send_int procId=%d\n",procId);
	
    if (procId == CORE_C674) {
		set_io(MAILBOX_MESSAGE_3,value);
		return 0;
    } else if (procId == CORE_VID) {
		set_io(MAILBOX_MESSAGE_4,value);
		return 0;
    } else if (procId == CORE_VPS) {
		set_io(MAILBOX_MESSAGE_5,value);
		return 0;
    	}else{
    		return -1;
    	}
}
#define DBGMEM_START_ADDR 0xa0000000
#define DBGMEM_END_ADDR   (0xa0000000 + 0x400000)
int cmem_dbgmem_init(u_int phy_dbgmem)
{
	int i;

	if((DBGMEM_START_ADDR > phy_dbgmem) || (phy_dbgmem > DBGMEM_END_ADDR)) 
	{
		printk("error phy_dbgmem %x\n", phy_dbgmem);
		return -1;
	}

	if(g_virt_dbgmem != NULL)
	{
		printk("dbgmem already mapped\n");
		return -1;
	}

	g_virt_dbgmem = ioremap(phy_dbgmem,DM740_NUMPROCS*0x10000);
	memset(g_virt_dbgmem,0,DM740_NUMPROCS*0x10000);
	printk("map phy addr 0x%x to virt addr 0x%x\n", phy_dbgmem, g_virt_dbgmem);

	for(i=0;i<DM740_NUMPROCS;i++) 
	{
		g_dbgmem[i] = g_virt_dbgmem + i*0x10000;
	}

	return 0;
}

