/*******************************************************************************
 *                                                                             *
 * Copyright 2008-2011 Hikvision Digital Technology Co., Ltd.                  *
 * FileName:     cpld.c                                                        *
 * Description:  cpld operation functions                                      *
 * History:                                                                    *
 *    VERSION	TIME	    AUTHOR		DESC                                   *
 *	  1.00      20081226    duhao       use for cpld control                   *
 *                                                                             *
 *******************************************************************************/
//#include <linux/config.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <asm/io.h>
#include "hikdvlib.h"

/*----------------------------------------------------------------------------------------------------*/
static cpld_info g_cpldinfo;
static int g_is_cpld_vaild = 0;
static u_char g_cpld_data[0x100];

/************************************************************************
 *
 * FUNC : get_cpld_data()
 * DESC : get hardware information from cpld
 * PARA : none
 *   
 *  RET : = 0
 *
 ************************************************************************/
static int get_cpld_data(void)
{
	u_char i,*cpldbase;

	
	cpldbase = (u_char *)ioremap(0x84000000,0x100);
	memcpy(g_cpld_data,cpldbase,sizeof(g_cpld_data));
	iounmap((void *)cpldbase);	

	//if(g_dbg) 
	if(1)
	{
		int i;
		for(i=0;i<sizeof(g_cpld_data);i++) {
			printk("%02x ",g_cpld_data[i]);
			if((i%16)==15) {
				printk("\n");
			}
		}
		printk("\n");
	}

	for(i=1;i<0x80;i++) {/* 128 is enough to tell cpld is there */
		if(g_cpld_data[i-1] != g_cpld_data[i]) {
			g_is_cpld_vaild = 1;
			break;
		}
	}
	

	/* set cpld information */
	if(g_is_cpld_vaild) {
		u_char *p;
		p = g_cpld_data;

		g_cpldinfo.cfgbit   |= 0x1;

		g_cpldinfo.flashCnt  = 1;
		g_cpldinfo.ethPort   = 1;
		g_cpldinfo.res[0]    = 0;/* pcb msb */
		g_cpldinfo.res[1]    = 0;/* pcb lsb */
		
		g_cpldinfo.devType   = p[0x6];
		g_cpldinfo.cpldDate  = p[0xa];
		
		g_cpldinfo.flashSize = (p[0x7] & 0x1) ? 32 : 16;		
		g_cpldinfo.flashType = (p[0x7] >> 1) & 0x1;
		g_cpldinfo.ddrType   = ((p[0x7] >> 2) & 0x3) ? 0 : 384 ;
		g_cpldinfo.ddrFreq   = (p[0x7] >> 4) & 0x3;
		g_cpldinfo.devFreq   = (p[0x7] >> 6) & 0x3;
		
		g_cpldinfo.idePort   = (p[0x8]) & 0x7;
		g_cpldinfo.netType   = (p[0x8] >> 3) & 0x1;
		g_cpldinfo.phyType   = (p[0x8] >> 4) & 0x3;
		g_cpldinfo.viType    = (p[0x8] >> 6) & 0x3;
				
		g_cpldinfo.sataType  = (p[0x9] >> 6) & 0x3;
		g_cpldinfo.res[6]    = (p[0x9] >> 5) & 0x1;/* 0=DVR|1=DVS */
		g_cpldinfo.res[2]    = (p[0x9]) & 0x7;/* alarm in */
		g_cpldinfo.res[3]    = (p[0x9] >> 3) & 0x3;/* arlarm out */

		g_cpldinfo.usbPort   = (p[0xb] >> 6) & 0x3;


		if(p[0xb] & (1<<5)) {
			g_cpldinfo.cfgbit   |= 0x2;
		}
	} else {
		printk("this board no cpld.\n");
	}
	
	return 0;
}


/************************************************************************
 *
 * FUNC : load_cpld()
 * DESC : get read hardware information from cpld
 * PARA : none
 * 
 *
 ************************************************************************/
void load_cpld(void)/* must only call one time */
{
	memset(g_cpld_data,0,sizeof(g_cpld_data));
	memset(&g_cpldinfo,0,sizeof(cpld_info));
	get_cpld_data();
}


/************************************************************************
 *
 * FUNC : cpld_reset_dev()
 * DESC : reset device by cpld
 * PARA : 
 *   devtyp - device id
 *
 ************************************************************************/
void cpld_reset_dev(int devtype)
{
	volatile u_char *cpldbase;
	if(!g_is_cpld_vaild)
		return;

	cpldbase = (u_char *)ioremap(0x84000000,0x100);
	/* reset dev */
	mdelay(20);
	cpldbase[5] = (~devtype);
	mdelay(100);
	cpldbase[5] = 0xff;
	mdelay(20);	
	iounmap((void *)cpldbase);
}
EXPORT_SYMBOL(cpld_reset_dev);



/************************************************************************
 *
 * FUNC : cpld_get_disk_count()
 * DESC : get disk count
 * PARA : none
 * return : 
 *		<0 is no disk
 *		>0 is disk count
 *
 ************************************************************************/
int cpld_get_disk_count(void)
{
	int diskcnt_map[] = {0,1,2,4,8};
	if(!g_is_cpld_vaild) {
		return -1;
	}
	return diskcnt_map[g_cpldinfo.idePort];
}

/************************************************************************
 *
 * FUNC : cpld_check_sii3114()
 * DESC : check sii3114 is exist
 * PARA : none
 * return : 
 *		=0 no sii3114
 *		1 sii3114 exist
 *
 ************************************************************************/
int cpld_check_sii3114(void)
{
	if(!g_is_cpld_vaild) {
		return 0;
	}

	if(g_cpldinfo.sataType == 1) {
		return 1;
	} else {
		return 0;
	}
	
}

/************************************************************************
 *
 * FUNC : cpld_get_boardtype()
 * DESC : get dev boardtype
 * PARA : none
 * return : 
 *		0xb000->0xc0000
 *		<=0 no cpld
 *
 ************************************************************************/
int cpld_get_boardtype(void)/* duhao add for new decoder borad */
{
	if(!g_is_cpld_vaild) {
		return 0;
	}
	return g_cpldinfo.devType;
}

/************************************************************************
 *
 * FUNC : cpld_is_dvs()
 * DESC : check device is DVS
 * PARA : none
 * return : 
 *		1 is DVS
 *		0 not DVS(maybe DVR or no cpld)
 *
 ************************************************************************/
int cpld_is_dvs(void)/* duhao add for dvs */
{
	if(!g_is_cpld_vaild) {
		return 0;
	}
	return g_cpldinfo.res[6] ? 1 : 0;
}

