//#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
//#include <asm/hardware.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/errno.h>
#include <asm/uaccess.h>
#include <linux/vmalloc.h>
#include <linux/sched.h>
#include <asm/io.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/syscalls.h>
#include <linux/dma-mapping.h>
#include <asm/cacheflush.h>
#include <linux/mm.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <plat/dma.h>
#include <asm/hardware/edma.h>
#include <linux/mman.h>

MODULE_LICENSE("hikvision");   

#define strtoul 					simple_strtoul
#define dbgprint(szFmt,args...)		{if(dbg) printk(szFmt,##args);}

static char *m_pszName = "hikvision netra dbgdrv";
static int dbg = 0;/* 0=debug off|1=debug on */
module_param(dbg,int,0644);
MODULE_PARM_DESC(dbg, "Show Debug Information");


#define HIK_DBG_MEM		0x10
#define HIK_MAP_IO		0x20
#define HIK_UNMAP_IO	0x30

#define MAX_BUF_SZ	256
typedef struct __dbg_param__ {
	u_int physaddr;
	u_int len;/* in bytes size */
	u_int rw;/* 0=read | !0=write */
	u_char buf[MAX_BUF_SZ];/* readbuf or writebuf */
} DBG_PARAM;


/* eg:
 *	 read  -- echo <reg> > /proc/dbg/iomem
 *   read  -- echo <reg> r<count> > /proc/dbg/iomem
 *	 write -- echo <reg> <val> > /proc/dbg/iomem 
 */
static ssize_t dbg_write(struct file *filp,const char __user *buf,unsigned long len,void *data)
{
	char *p = NULL;
	u_char input[256];
	u_int val = 0,phyreg,virtreg,rcnt=0;
	
	if (len >= 256) {
		return -ENOMEM;	
	}
	if (copy_from_user(input,buf,len )) {
		return -EFAULT;
	}

	if(input[len-1]=='\n') {
		input[len-1]=0;
	} else {
		input[len]=0;
	}

	
	p = strchr(input,' ');
	if(p) {
		if(p[1]=='r') {
			rcnt = simple_strtoul(p+2,NULL,16);
		} else {
			if(p[1]!=0) {
				val = simple_strtoul(p+1,NULL,16);
				*p = 0;
			} else {
				rcnt = 1;
			}
		}
	} else {
		rcnt = 1;
	}
	phyreg = simple_strtoul(input,NULL,16);
	
	{
		if(rcnt) {
			virtreg = (u_int)ioremap(phyreg,4*rcnt);
		} else {
			virtreg = (u_int)ioremap(phyreg,4);/* write */
		}		
	}
	
	if(p && !rcnt) {
		printk("[0x%08X]0x%08X",virtreg,*((volatile u_int *)virtreg));
		*((volatile u_int *)virtreg) = val;
		printk("-->0x%08X\n",val);
	} else {
		int i;
		for(i=0;i<rcnt;i++) {
			printk("[0x%08X]0x%08X\n",virtreg+4*i,*((volatile u_int *)(virtreg+4*i)));	
		}
	}

	iounmap((void *)virtreg);
	return len;
}

static int dbgdrv_mmap(struct file *file, struct vm_area_struct *vma)
{
	int size = vma->vm_end-vma->vm_start;
	//vma->vm_flags |= VM_IO;
	//vma->vm_flags |= VM_RESERVED;
	vma->vm_page_prot = 0x183|L_PTE_MT_WRITEBACK;
	if( remap_pfn_range(vma,vma->vm_start,vma->vm_pgoff,size,vma->vm_page_prot) ) {
		printk("remap_pfn_range failed map memory[0x%08lX,0x%08X] to user space.\n",vma->vm_start,size);
		return -EAGAIN;
	}
	return 0;
}

static int dbgdrv_ioctl(struct file *file, u_int cmd, u_long arg)
{
	switch(cmd) {
		case HIK_DBG_MEM:
		{
			int i;
			char *virtaddr;
			DBG_PARAM dp;
			copy_from_user(&dp,(void *)arg,sizeof(dp));

			if( (dp.physaddr & 3) || (dp.len & 3) || (dp.len>MAX_BUF_SZ) ) {
				printk("invaild param\n");
				return -EINVAL;
			}

			virtaddr = (char *)ioremap(dp.physaddr,dp.len);
			if(virtaddr==NULL) {
				printk("remap physaddr=0x%x,len=0x%x failed\n",dp.physaddr,dp.len);
				return -ENOMEM;
			}
			
			if(dp.rw) {/* write */
				for(i=0;i<dp.len;i+=4) {
					*(volatile u_int *)(virtaddr + i) = *(volatile u_int *)(dp.buf + i);
				}
			} else {/* read */
				for(i=0;i<dp.len;i+=4) {
					*(volatile u_int *)(dp.buf + i) = *(volatile u_int *)(virtaddr + i);
				}
			}

			copy_to_user((void *)arg,&dp,sizeof(dp));
			break;
		}
		case HIK_MAP_IO:
		{
			u_long virtaddr,map_info[2];
			u_long physaddr,size; 
			if(arg == 0)
				return -EINVAL;

			copy_from_user((void *)map_info, (void *)arg, 2 * sizeof(u_long));
			physaddr = ( (map_info[0] + PAGE_SIZE - 1) / PAGE_SIZE ) * PAGE_SIZE;
			size  = ( map_info[1] / PAGE_SIZE ) * PAGE_SIZE;
			if( size == 0 ) {/* phys size */
				return -EINVAL;	
			}
			if(physaddr != map_info[0] || size != map_info[1]) {
				return -EINVAL;
			}

			virtaddr = (u_long)ioremap_nocache(physaddr,size);
			if(virtaddr==0) {
				printk("ioremap phys memery[0x%08lX,0x%08lX] failed\n",physaddr,size);
				return -ENODEV;
			}
			
			down_write(&current->mm->mmap_sem);
			map_info[0] = do_mmap_pgoff(file,virtaddr,size,PROT_READ|PROT_WRITE,MAP_SHARED,physaddr>>PAGE_SHIFT);
			up_write(&current->mm->mmap_sem);

			if(map_info[0] < 0) {
				iounmap((void *)virtaddr);
				printk("do_mmap_pgoff failed.\n");
				return -ENOMEM;
			}

			copy_to_user((void *)arg, (void *)map_info, 2 * sizeof(u_long));

			break;
		}
		case HIK_UNMAP_IO:
		{
			u_long map_info[2];
			u_long virtaddr,size; 
			if(arg == 0)
				return -EINVAL;

			copy_from_user((void *)map_info, (void *)arg, 2 * sizeof(u_long));
			virtaddr = ( (map_info[0] + PAGE_SIZE - 1) / PAGE_SIZE ) * PAGE_SIZE;
			size  = ( map_info[1] / PAGE_SIZE ) * PAGE_SIZE;
			if( size == 0 ) {
				return -EINVAL;
			}
			if(virtaddr != map_info[0] || size != map_info[1]) {
				return -EINVAL;
			}

			//munmapͷ
			down_write(&current->mm->mmap_sem);
			do_munmap(current->mm, virtaddr, size);		
			up_write(&current->mm->mmap_sem);

			break;
		}
		default:
			return -EINVAL;
	}
	return 0;	
}

static struct file_operations dbgdrv_fops = {
	.owner = THIS_MODULE,
	.unlocked_ioctl = dbgdrv_ioctl,
	.mmap = dbgdrv_mmap
};


int __init dbgdrv_init(void)
{	

	struct proc_dir_entry *pentry;
	
	printk("dbgdrv=%d,Name=%s\n",dbg,m_pszName);

	proc_mkdir("dbgdrv",NULL);
	pentry = create_proc_entry("dbgdrv/dump",0,NULL);
	pentry->write_proc = dbg_write;

    if (register_chrdev(230,"dbgdrv",&dbgdrv_fops) < 0) {
    	printk("register dbgdrv driver failed.\n");
		return -ENODEV;
    }
	return 0;
}



void __exit dbgdrv_exit(void)
{
	unregister_chrdev(230,"dbgdrv");

	remove_proc_entry("dbgdrv/dump",NULL);
	remove_proc_entry("dbgdrv",NULL);		
	printk("exit dbgdrv\n");
}

module_init(dbgdrv_init); 
module_exit(dbgdrv_exit);

