//#include <linux/config.h>
//#include <linux/version.h>
#include <linux/module.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/errno.h>
#include <asm/uaccess.h>
#include <linux/vmalloc.h>
#include <linux/sched.h>
#include <asm/io.h>
//#include <linux/semaphore.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/i2c.h>
//#define strtoul 						simple_strtoul

MODULE_LICENSE("hikvision");   

#define EEPROM_I2C_ADDR		(0xa6)
#define EEPROM_SIZE			512*128/* 64k */
#define EEPROM_PAGE			128/* byte */

static int g_eeprom_init = 0;
static struct i2c_client eeprom_i2c_client;
//static struct i2c_driver eeprom_i2c_driver; 

extern u_char gpio_eeprom_random_read(u_char devaddr,u_short addr);
extern void gpio_eeprom_write(u_char devaddr,u_short addr,u_char *buf,int len);

static int g_offset = 0;

static int eeprom_i2c_read(struct i2c_client *client, u8 *buf,int len)
{
	int i;
	for(i=0;i<len;i++) {
		buf[i] = gpio_eeprom_random_read(EEPROM_I2C_ADDR,g_offset + i);
	}
	return len;
}

static int eeprom_i2c_seek(struct i2c_client *client, u16 addr)
{	
	gpio_eeprom_random_read(EEPROM_I2C_ADDR,addr);
	return 2;
}

static int eeprom_i2c_byte_write(struct i2c_client *client, u16 addr, u8 val)
{
	gpio_eeprom_write(EEPROM_I2C_ADDR,addr,&val,1);
	return 3;	
}

static int eeprom_i2c_page_write(struct i2c_client *client, u16 addr, u8 *buf)/* 128 byte */
{
	gpio_eeprom_write(EEPROM_I2C_ADDR,addr,buf,EEPROM_PAGE);
	return EEPROM_PAGE+2; 
}

static int eeprom_open(struct inode *inode, struct file *filp)
{
	if( !g_eeprom_init)
		return -ENODEV;	
	eeprom_i2c_seek(&eeprom_i2c_client,0);
	g_offset = 0;
	return 0;
}

ssize_t eeprom_write(struct file *filp, const char __user *buf, size_t size, loff_t *pos)
{
	int i,j,left,pagecnt;
	int hdrbytes;/* the bytes not in page align */
	char *buffer;

	if( !g_eeprom_init)
		return -ENODEV;	
	
	buffer = (char *)__get_free_page(GFP_KERNEL);
	if (!buffer)
		return -ENOMEM;

	//eeprom_i2c_seek(&eeprom_i2c_client,g_offset);

	if(g_offset%EEPROM_PAGE)/* write pos not page align */
	{
		hdrbytes = EEPROM_PAGE - (g_offset%EEPROM_PAGE);
		if(hdrbytes>size)/* write data less one page */
		{
			hdrbytes = size;
		}
	}
	else/* write pos page align */
	{
		hdrbytes = 0;
	}
	if(hdrbytes)
	{
		/* duhao fix when eeprom write not page align bug 
		 *   ervery eeprom write operation cannot exceed one page size,
		 *   or then exceed bytes will recycle to the page start address
		 */
		copy_from_user(buffer,buf,hdrbytes);
		for(j=0;j<hdrbytes;j++)
		{
			if(g_offset>=EEPROM_SIZE)/* overflow eeprom size throw some bytes of last */
			{
				free_page((u_long)buffer);	
				return j;
			}
			eeprom_i2c_byte_write(&eeprom_i2c_client,g_offset++,buffer[j]);
		}
	}
	if(hdrbytes == size)
	{
		free_page((u_long)buffer);	
		return size;
	}
	
	size -= hdrbytes;
	
	pagecnt = size/EEPROM_PAGE;
	for(i=0;i<pagecnt;i++)
	{
		copy_from_user(buffer,buf + hdrbytes + i*EEPROM_PAGE,EEPROM_PAGE);
		if(g_offset>(EEPROM_SIZE-EEPROM_PAGE))
		{
			free_page((u_long)buffer);	
			return hdrbytes + i*EEPROM_PAGE;
		}
		eeprom_i2c_page_write(&eeprom_i2c_client,g_offset,buffer);
		g_offset+=EEPROM_PAGE;
	}

	left = size%EEPROM_PAGE;
	if(left)
	{
		copy_from_user(buffer,buf + hdrbytes + pagecnt*EEPROM_PAGE,left);
		for(j=0;j<left;j++)
		{
			if(g_offset>=EEPROM_SIZE)
			{
				free_page((u_long)buffer);	
				return hdrbytes + pagecnt*PAGE_SIZE + j;
			}
			eeprom_i2c_byte_write(&eeprom_i2c_client,g_offset++,buffer[j]);
		}
	}

	free_page((u_long)buffer);	
	return hdrbytes + size;
}

ssize_t eeprom_read(struct file *filp,char __user *buf, size_t size, loff_t *pos)
{
	int i,left,pagecnt;
	char *buffer;

	if( !g_eeprom_init)
		return -ENODEV;	

	buffer = (char *)__get_free_page(GFP_KERNEL);
	if (!buffer)
		return -ENOMEM;

	//eeprom_i2c_seek(&eeprom_i2c_client,g_offset);	

	pagecnt = size/PAGE_SIZE;

	for(i=0;i<pagecnt;i++)
	{	
		if(g_offset+PAGE_SIZE>EEPROM_SIZE)
		{
			free_page((u_long)buffer);	
			return i*PAGE_SIZE;
		}
		eeprom_i2c_read(&eeprom_i2c_client,buffer,PAGE_SIZE);
		copy_to_user(buf+i*PAGE_SIZE,buffer,PAGE_SIZE);
		g_offset += PAGE_SIZE;
	}
	
	
	left = size%PAGE_SIZE;
	if(left)
	{
		if(g_offset+left>EEPROM_SIZE)
		{
			free_page((u_long)buffer);	
			return pagecnt*PAGE_SIZE;
		}
		eeprom_i2c_read(&eeprom_i2c_client,buffer,left);
		copy_to_user(buf+pagecnt*PAGE_SIZE,buffer,left);
		g_offset += left;
	}

	free_page((u_long)buffer);	
	return size;
}

loff_t eeprom_seek(struct file *filp, loff_t offset, int flag)
{
	if( !g_eeprom_init)
		return -ENODEV;	

	if(flag!=0)
		return -EINVAL;
	if(offset>EEPROM_SIZE)
		return -EINVAL;
	eeprom_i2c_seek(&eeprom_i2c_client,offset);
	g_offset = offset;
	return offset;
}

int eeprom_close(struct inode *inode, struct file *filp)
{
	return 0;
}

static struct file_operations eeprom_fops = {
	open:		eeprom_open,
	read:		eeprom_read,
	write:		eeprom_write,
	llseek:		eeprom_seek,
	release:	eeprom_close,
};

static int eeprom_major;
u_char g_vo_mode = 0;/* 1=ntsc|!1=pal use for davincifb.c */
extern u_char g_rs232_mode;/* 1=trans chan|!1=debug */
int __init eeprom_init(void)
{
	int ret;

	eeprom_major = register_chrdev(240, "eeprom", &eeprom_fops);
	if (eeprom_major<0) 
	{
        printk("Failed to allocate major number.\n");
		return -ENODEV;
    }

/* check hardware is connected */
#if 1
{
	u_int magic = 0;

	eeprom_i2c_seek(&eeprom_i2c_client,0);
	ret = eeprom_i2c_read(&eeprom_i2c_client,(u8 *)&magic,4);
	if( ret>0 )/* duhao modify for hd i2c driver diff */
	{
		u_char buf[2];
		g_eeprom_init = 1;
		eeprom_i2c_seek(&eeprom_i2c_client,EEPROM_SIZE-2);
		eeprom_i2c_read(&eeprom_i2c_client,buf,2);
		printk("eeprom driver init,magic=0x%x,vo_mode=0x%02x,tc_mode=0x%02x\n",
			magic,buf[1],buf[0]);
		if(magic == 0x484b5753) {/* "HKWS" */
			g_rs232_mode = buf[0];
			g_vo_mode = buf[1];
		}
	}
	else
	{
		g_eeprom_init = 0;
		printk("eeprom: hardware not connected,ret=%d\n",ret);
	}
}
#endif

	return 0;
}


void __exit eeprom_exit(void)
{		
	unregister_chrdev(240,"eeprom");
	printk("eeprom driver exit\n");
}


module_init(eeprom_init); 
module_exit(eeprom_exit);


