#include <linux/kernel.h>
#include <linux/blkdev.h>
#include <linux/spinlock.h>
#include <scsi/scsi.h>
#include <scsi/scsi_cmnd.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_device.h>
#include <scsi/scsi_driver.h>
#include <linux/libata.h>
#include <asm/uaccess.h>
//#include <asm/arch/hikdvlib.h>
#include <linux/proc_fs.h>
#include <linux/hdreg.h>

#define SCSI_VENDOR_COMMAND_TRANS 	0x77
#define ATA_MAX_PARTS   				4
#define MAX_DISK						8

#ifndef ERR_STAT
/* Bits of HD_STATUS */
#define ERR_STAT		0x01
#define INDEX_STAT		0x02
#define ECC_STAT		0x04	/* Corrected error */
#define DRQ_STAT		0x08
#define SEEK_STAT		0x10
#define SRV_STAT		0x10
#define WRERR_STAT		0x20
#define READY_STAT		0x40
#define BUSY_STAT		0x80

/* Bits for HD_ERROR */
#define MARK_ERR		0x01	/* Bad address mark */
#define TRK0_ERR		0x02	/* couldn't find track 0 */
#define ABRT_ERR		0x04	/* Command aborted */
#define MCR_ERR		0x08	/* media change request */
#define ID_ERR			0x10	/* ID field not found */
#define MC_ERR			0x20	/* media changed */
#define ECC_ERR		0x40	/* Uncorrectable ECC error */
#define BBD_ERR		0x80	/* pre-EIDE meaning:  block marked bad */
#define ICRC_ERR		0x80	/* new meaning:  CRC error during transfer */
#endif

#define SCSI_IS_READ(cmd)		\
	( ((cmd) == READ_6) ||		\
	  ((cmd) == READ_10) ||		\
	  ((cmd) == READ_12) ||		\
	  ((cmd) == READ_16))

#define SCSI_IS_WRITE(cmd)	\
	( ((cmd) == WRITE_6) ||	\
	  ((cmd) == WRITE_10) ||	\
	  ((cmd) == WRITE_12) ||	\
	  ((cmd) == WRITE_16) )
	  
#define SCSI_IS_VERIFY(cmd)		(((cmd) == VERIFY) || ((cmd) == VERIFY_16))

#define among(a,b,c)   			(((a)<=(b))&&((b)<=(c)))



typedef union {
	struct {
#if defined (__LITTLE_ENDIAN)
		u32 low;
		u32 high;
#elif defined (__BIG_ENDIAN)
		u32 high;
		u32 low;
#endif
        } parts;
	u64 value;
} SIL_U64, *PSIL_U64, *SIL_PU64;


typedef struct {
   /* hdisk part info  if dospt_absSec=0xffffffff mean	no partition*/
	u64		dosfs_dirSec[ATA_MAX_PARTS];/* starting dir area 0:not protect */
	u64		dosfs_absSec[ATA_MAX_PARTS];/* starting absolute sector */
   	u64		dosfs_nSectors[ATA_MAX_PARTS];/* number of sectors in part */
	u8		bCheckKeyArea;
} DEVICE_PARTITION_INFO;  

typedef struct _hdd_exc_
{
	void *dev; /* kernel use, override by magic in ioctl */
	u32 readErr;
	u32 writeErr;
	u32 keyAreaErr; /*lba in key area*/
	u32 lbaOverflow; /*lba out of range*/
	u32 NoReady; /*disk no ready*/
	u32 BSYCount; /*disk busy*/
	u32 DRQSetErr; /*DRQ not set*/
	u32 hdStatus; /* disk status: idle error exist */

	u32 retryCmdCount;
	u32 abortCmdCount; /* host initiative */
	u32 CmdAborted; /* passivity */
	u32 CmdOverUnknown;
	u32 CmdOverErr;

	u32 BadSCSICmdParam; /* scsi comand parameter bad */
	u32 UAReset; /* unit attention bus reset */
	u32 UAParamChanged; /* unit attention parameter changed */
	u32 QueueFull; /* scsi comand queue full */
	u32 OverRun; /* edma reg SerrInt*/
	u32 UnderRun;
	u32 ParityErr;
	u32 BadSCSICmdResult;

	u32 DeviceErr; /* Neither device error without completion nor self disable must be in NCQ
					* mode and Port multiplier connected*/
	u32 RecoverableErr;
	u32 SerrInt; /* one bit set in SError*/
	u32 LinkCtlRxErr;
	u32 LinkCtlTxErr;
	struct timespec diskAccessTime;
	u32 reserved1;
	u32 reserved2;
} hdd_exc;


static DEVICE_PARTITION_INFO g_sec_keyarea[MAX_DISK];
int g_protect_keyarea[MAX_DISK];
hdd_exc g_hdd_excs[MAX_DISK];/* max disk is 8 */

static void SetLBAandSectorCount(struct scsi_cmnd *cmd, SIL_U64 *pLBA, u32 *pSectorCount)
{
	SIL_U64 tmpLBA;
	u32 tmpSectorCount;
	u8 *cmnd = cmd->cmnd;
	
	if ((!SCSI_IS_READ(cmnd[0])) && 
	    (!SCSI_IS_WRITE(cmnd[0])) && 
	    (!SCSI_IS_VERIFY(cmnd[0])))
		return;
	
	/* This is READ/WRITE command */
	switch (cmnd[0]) {
	case READ_6:
	case WRITE_6:
		tmpLBA.value = (u32)((((u32)(cmnd[1] & 0x1F))<<16) |
					((u32)cmnd[2]<<8) |
					((u32)cmnd[3]));
		tmpSectorCount = (u32)cmnd[4];
		break;
	case READ_10:
	case WRITE_10:
	case VERIFY:
		tmpLBA.value = (u32)(((u32)cmnd[2]<<24) | 
					((u32)cmnd[3]<<16) | 
					((u32)cmnd[4]<<8) | 
					((u32)cmnd[5]));
		tmpSectorCount = ((u32)cmnd[7]<<8) | (u32)cmnd[8];
		break;
	case READ_12:
	case WRITE_12:
		tmpLBA.value = (u32)(((u32)cmnd[2]<<24) | 
					((u32)cmnd[3]<<16) | 
					((u32)cmnd[4]<<8) | 
					((u32)cmnd[5]));
		tmpSectorCount = (u32)(((u32)cmnd[6]<<24) |
					((u32)cmnd[7]<<16) |
					((u32)cmnd[8]<<8) |
					((u32)cmnd[9]));
		break;
	case READ_16:
	case WRITE_16:
	case VERIFY_16:
		tmpLBA.parts.high = (u32)(((u32)cmnd[2]<<24) | 
					((u32)cmnd[3]<<16) | 
					((u32)cmnd[4]<<8) | 
					((u32)cmnd[5]));
		tmpLBA.parts.low = (u32)(((u32)cmnd[6]<<24) |
					((u32)cmnd[7]<<16) |
					((u32)cmnd[8]<<8) |
					((u32)cmnd[9]));

		tmpSectorCount = (u32)(((u32)cmnd[10]<<24) |
					((u32)cmnd[11]<<16) |
					((u32)cmnd[12]<<8) |
					((u32)cmnd[13]));
		break;
	default:
		printk("Unsupported READ/WRITE command [%x]\n", cmnd[0]);
		tmpLBA.value = 0;
		tmpSectorCount = 0;
	}
	*pLBA = tmpLBA;
	*pSectorCount = tmpSectorCount;
}

int get_host_no(struct scsi_cmnd *cmd)
{
	struct scsi_device *sdev;
	struct Scsi_Host *host;
	sdev = cmd->device;
	host = sdev->host;
	return host->host_no;
}

static int IsLBAInKeyArea(struct scsi_cmnd *cmd, struct ata_device *dev, SIL_U64 tmpLBA)
{
	int i,idx;
	u64 startSector;	
	DEVICE_PARTITION_INFO *pPartInfo;

	idx = get_host_no(cmd);
	pPartInfo = &g_sec_keyarea[idx];

	//startSector = ((tmpLBA.parts.high) << 32) | (tmpLBA.parts.low);
	startSector = tmpLBA.parts.high;
	startSector <<= 32;
	startSector |= tmpLBA.parts.low;
	
    for(i= 0; i < ATA_MAX_PARTS; i++) {/* ATA_MAX_PARTS */
		if(among(pPartInfo->dosfs_absSec[i], startSector, pPartInfo->dosfs_dirSec[i])) {/* check LBA if in key area */
	   		if(pPartInfo->bCheckKeyArea && SCSI_IS_WRITE(cmd->cmnd[0])) {/* if protect key area */
				printk("<0>scsi%d : LBA %#llx not in partition[%d] keyarea[%#llx-->%#llx]\n",/* LBA in key area return true and alarm */
					idx, startSector, i,
					pPartInfo->dosfs_absSec[i], pPartInfo->dosfs_dirSec[i]);
				g_hdd_excs[idx].keyAreaErr++;
	       		return 1;         
         	}
		}
	}
    return 0;
}

static void set_protected_keyearea(struct ata_device *dev, u8 *pdata, int idx)
{
	DEVICE_PARTITION_INFO *sec;
	u8 *buff = pdata;
	u8 part, DataBufOffset;

	sec = &g_sec_keyarea[idx];
	
	if (buff[1]) {
   	 	sec->bCheckKeyArea = 1;	/* protect key area */
		g_protect_keyarea[idx] = 1;
   	} else {	
   	 	sec->bCheckKeyArea= 0;	/* not protect key area */
		g_protect_keyarea[idx] = 0;
   	}
	
	for (part = 0; part < ATA_MAX_PARTS; part++) {	 /* set  absSec and dirSec */
   	        
		DataBufOffset = 16 * (part + 1);
       	sec->dosfs_absSec[part] =  (((u64) buff[DataBufOffset]) << 0) |
        	(((u64) buff[DataBufOffset+1]) << 8) |
        	(((u64) buff[DataBufOffset+2]) << 16) |
        	(((u64) buff[DataBufOffset+3]) << 24) |
        	(((u64) buff[DataBufOffset+4]) << 32) |
        	(((u64) buff[DataBufOffset+5]) << 40) |
        	(((u64) buff[DataBufOffset+6]) << 48) |
        	(((u64) buff[DataBufOffset+7]) << 56);                                              /* 64bit absSec */
	   
		sec->dosfs_dirSec[part] = (((u64) buff[DataBufOffset+8]) << 0) |
        	(((u64) buff[DataBufOffset+9]) << 8) |
        	(((u64) buff[DataBufOffset+10]) << 16) |
        	(((u64) buff[DataBufOffset+11]) << 24) |
        	(((u64) buff[DataBufOffset+12]) << 32) |
        	(((u64) buff[DataBufOffset+13]) << 40) |
        	(((u64) buff[DataBufOffset+14]) << 48) |
        	(((u64) buff[DataBufOffset+15]) << 56);                                              /* 64bit dirSec */
	
		printk(KERN_EMERG "dosfs_absSec[%d] %#llx, dosfs_dirSec[%d] %#llx, bCheckKeyArea %d\n", 
			part,  sec->dosfs_absSec[part], part, sec->dosfs_dirSec[part], buff[1]);
   	}
}

int set_key_area(struct scsi_cmnd *cmd, struct ata_device *dev)
{
	struct scatterlist *sg;
	u8 *pdata;

	printk("protected keyarea\n");

	//sg = (struct scatterlist *)cmd->request_buffer;
	sg = (struct scatterlist *)cmd->sdb.table.sgl;

	pdata = (u8 *)kmalloc(sg->length,GFP_ATOMIC);
	if ( pdata == NULL) {
		printk("queuecommand: Failed to allocate data buffer for kmap\n");
		cmd->result = SAM_STAT_BUSY;
		return 0;
	}

	if( cmd->sc_data_direction == DMA_TO_DEVICE) {
		u8 *buf;
		buf = kmap_atomic(sg_page(sg), KM_USER0) + sg->offset;
		memcpy(pdata , buf , sg->length);
		kunmap_atomic(buf - sg->offset, KM_USER0);

		if( pdata[0] == SCSI_VENDOR_COMMAND_TRANS ) {	
			set_protected_keyearea(dev,pdata,get_host_no(cmd));
			cmd->result = SAM_STAT_GOOD;
			return 0;
		}
	}
	kfree((void *)pdata);
	return -1;/* may be other cmd go through */
}

int check_key_area(struct scsi_cmnd *cmd, struct ata_device *dev)
{
	SIL_U64 LBA;	
	u32 SectorCount;

#if 0/* dump write sector data */
{
	struct scatterlist *sg = (struct scatterlist *)cmd->request_buffer;
	if(sg && sg->length>=512 && SCSI_IS_WRITE(cmd->cmnd[0]) ) {
		char *pdata = (u8 *)kmalloc(sg->length,GFP_ATOMIC);
		if ( pdata ) {
			u8 *buf;
			int i;
			printk("sg->offset=%d,sg->length=%d\n",sg->offset,sg->length);
			buf = kmap_atomic(sg_page(sg), KM_USER0) + sg->offset;
			memcpy(pdata , buf , sg->length);
			kunmap_atomic(buf - sg->offset, KM_USER0);

			for(i=0;i<sg->length;i++) {
				if(i%16==0)
					printk("\n");
				printk("%02x ",pdata[i]);
			}
			printk("\n");
			kfree((void *)pdata);
		}
	}
}
#endif

	SetLBAandSectorCount(cmd, &LBA, &SectorCount);	
	if(IsLBAInKeyArea(cmd, dev, LBA)) {
		printk("LBA is in KeyArea!\n");
		cmd->result |= DID_ABORT << 16;
		return 1;
	} 
	return 0;/* not in key area */
}

void init_key_area(void)
{
	memset(g_hdd_excs,0,sizeof(hdd_exc));
	memset(g_sec_keyarea,0,sizeof(g_sec_keyarea));
	memset(g_protect_keyarea,0,sizeof(g_protect_keyarea));
}

/***********************************************************************************************************************/
#define HD_STATUS_ERROR     0x1
#define HD_STATUS_NOEXIST   0x2 /* EXIST - 0, NOEXIST - 1 */
#define HD_STATUS_IDLE      0x4  

static struct proc_dir_entry *proc_sata_dir = NULL;
static struct proc_dir_entry *proc_sataport = NULL;

void check_drive_present(int idx)
{
	int present;
	struct ata_port *ap = g_hdd_excs[idx].dev;
	if(ap == NULL) {
		return;
	}
	
	present = ata_link_online(&ap->link);
	if (present) {
		g_hdd_excs[idx].hdStatus &= (~HD_STATUS_NOEXIST);
	} else {
		g_hdd_excs[idx].hdStatus |= HD_STATUS_NOEXIST;
	}
}

#if 0
static unsigned long long sectors_to_MB(unsigned long long n)
{
	n <<= 9;		/* make it bytes */
	do_div(n, 1000000);	/* make it MB */
	return n;
}
#endif

static int proc_read_sataport(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	struct ata_port *ap = (struct ata_port *)data;
	int idx = ap->port_no;
	hdd_exc *e = &g_hdd_excs[idx];
	char *out = page;
	int len;

	check_drive_present(idx);
#if 0
	out += sprintf(out, " serial no.          %.20s\n",
		(id && id->serial_no[0]) ? (char *)id->serial_no : "(none)");
	out += sprintf(out, " model               %.40s\n",
		(id && id->model[0]) ? (char *)id->model : "(none)");
	out += sprintf(out, " firmware            %.8s\n",
		(id && id->fw_rev[0]) ? (char *)id->fw_rev : "(none)");
	out += sprintf(out, " capacity            %lld MB\n",
		sectors_to_MB((DRIVER(drive)->capacity(drive))));
#endif
	out += sprintf(out, "\n");

	out += sprintf(out, " lbaOverflow         %d\n", e->lbaOverflow);
	out += sprintf(out, " keyAreaErr          %d\n", e->keyAreaErr);
	out += sprintf(out, " BSYCount            %d\n", e->BSYCount);
	out += sprintf(out, " NoReady             %d\n", e->NoReady);
	out += sprintf(out, " DRQSetErr           %d\n", e->DRQSetErr);
	out += sprintf(out, " writeErr            %d\n", e->writeErr);
	out += sprintf(out, " readErr             %d\n", e->readErr);
	out += sprintf(out, " retryCmdCount       %d\n", e->retryCmdCount);
	out += sprintf(out, " abortCmdCount       %d\n", e->abortCmdCount);
	out += sprintf(out, " CmdAborted          %d\n", e->CmdAborted);
	out += sprintf(out, " CmdOverUnknown      %d\n", e->CmdOverUnknown);
	out += sprintf(out, " CmdOverErr          %d\n", e->CmdOverErr);
	
	out += sprintf(out, " hdStatus            %s  %s  %#x\n",
		((e->hdStatus & HD_STATUS_ERROR) || (e->hdStatus & HD_STATUS_NOEXIST)) ? "ABNORMAL" : "NORMAL",
		(e->hdStatus & HD_STATUS_IDLE) ? "IDLE" : "ACTIVE", e->hdStatus);
	out += sprintf(out, " BadSCSICmdParam     %d\n", e->BadSCSICmdParam);
	out += sprintf(out, " UAReset             %d\n", e->UAReset);
	out += sprintf(out, " UAParamChanged      %d\n", e->UAParamChanged);
	out += sprintf(out, " QueueFull           %d\n", e->QueueFull);
	out += sprintf(out, " OverRun             %d\n", e->OverRun);
	out += sprintf(out, " UnderRun            %d\n", e->UnderRun);
	out += sprintf(out, " ParityErr           %d\n", e->ParityErr);
	out += sprintf(out, " BadSCSICmdResult    %d\n", e->BadSCSICmdResult);
		
	out += sprintf(out, " RecoverableErr      %d\n", e->RecoverableErr);
	out += sprintf(out, " SerrInt             %d\n", e->SerrInt);
	out += sprintf(out, " LinkCtlRxErr        %d\n", e->LinkCtlRxErr);
	out += sprintf(out, " LinkCtlTxErr        %d\n", e->LinkCtlTxErr);
	/* TODO deviceErr? */

	out += sprintf(out, " accessTime          %ld\n", (long)e->diskAccessTime.tv_sec);
	out += sprintf(out, "\n");

#if 0	
	if (s->enable) { /* protect key area */
		int i;
		out += sprintf(out, " keyArea enabled (range in hex):\n");
		for (i = 0; i < ATA_MAX_PARTS; i++) {
			out += sprintf(out, " Partition %d: protected sectors [ %8lx,  %8lx ]\n",
				i, s->beg[i], s->end[i]);
		}
	} else {
		out += sprintf(out, " keyArea disabled!!!\n");    
	}
#endif
	if(g_sec_keyarea[idx].bCheckKeyArea) {
		int i;
		out += sprintf(out, " keyArea enabled (range in hex):\n");
		for (i = 0; i < ATA_MAX_PARTS; i++) {
			out += sprintf(out, " Partition %d: protected sectors [ %8lx, %8lx ]\n",i, 
				g_sec_keyarea[idx].dosfs_dirSec[i], 
				g_sec_keyarea[idx].dosfs_absSec[i]);
		}
	} else {
		out += sprintf(out, " keyArea disabled!!!\n");    
	}
	out += sprintf(out, "\n");

	len = out - page;

	//PROC_IDE_READ_RETURN(page,start,off,count,eof,len);
	len -= off;
	if (len < count) {
		*eof = 1;
		if (len <= 0)
			return 0;
	} else
		len = count;
	*start = page + off;
	return len;
	
}

void hik_set_proc(struct ata_port *ap,int idx)
{

	if(!proc_sata_dir) {
		proc_sata_dir = proc_mkdir("sata", NULL);
	}
	char portname[20];
		
	sprintf(portname, "sataport%d", idx);
	proc_sataport = create_proc_entry(portname, S_IFREG|S_IRUSR|S_IWUSR, proc_sata_dir);
	if(proc_sataport){
		proc_sataport->data = (void *)ap;
		proc_sataport->read_proc = proc_read_sataport;
		proc_sataport->write_proc = NULL;
	}

	g_hdd_excs[idx].dev = (void *)ap;
}

void hik_set_hdd_exc(unsigned id,u_char stat,u_char err,u_int flags)
{
	int idx = id - 1;

	if (stat & BUSY_STAT) {
		g_hdd_excs[idx].BSYCount++;	
	} else {
		if (stat & READY_STAT) {
		} else {
			g_hdd_excs[idx].NoReady++;
		}
		if (stat & DRQ_STAT) {
			g_hdd_excs[idx].DRQSetErr++;
		}
		if (stat & ECC_STAT) {
			g_hdd_excs[idx].RecoverableErr++;
		}
	}

	if (err) {
		if( flags == ATA_TFLAG_WRITE ){
			g_hdd_excs[idx].writeErr++;
		} else {
			g_hdd_excs[idx].readErr++;
		}

		if (err & ABRT_ERR) {
			g_hdd_excs[idx].abortCmdCount++;
		}
		if (err & ICRC_ERR) {
			g_hdd_excs[idx].ParityErr++;
		}
		if (err & ECC_ERR) {
			g_hdd_excs[idx].DeviceErr++;
		}
		if (err & ID_ERR) {
			g_hdd_excs[idx].lbaOverflow++;
		}
	}
}

void hik_set_hdd_active(int idx,int is_active)
{
	if(is_active) {
		g_hdd_excs[idx].hdStatus &= (~HD_STATUS_IDLE);
		g_hdd_excs[idx].diskAccessTime = current_kernel_time();
	} else {
		g_hdd_excs[idx].hdStatus |= HD_STATUS_IDLE;
	}
}

void hik_get_hdd_exc(int idx, void *buf)
{
	check_drive_present(idx);
	copy_to_user(buf,&g_hdd_excs[idx], sizeof(hdd_exc));
}
