#ifndef __HIKDVLIB_H__
#define __HIKDVLIB_H__

#define GPIO0_BASE			TI81XX_L4_SLOW_IO_ADDRESS(0x48032000)
#define GPIO1_BASE			TI81XX_L4_SLOW_IO_ADDRESS(0x4804c000)
#define GPIO2_BASE			TI81XX_L4_SLOW_IO_ADDRESS(0x48032000)
#define GPIO3_BASE			TI81XX_L4_SLOW_IO_ADDRESS(0x4804c000)

#define GPIO_OE			(GPIO_BASE + 0x134)
#define GPIO_DATAIN		(GPIO_BASE + 0x138)
#define GPIO_DATAOUT	(GPIO_BASE + 0x13c)
#define GPIO_CLROUT		(GPIO_BASE + 0x190)
#define GPIO_SETOUT		(GPIO_BASE + 0x194)

#define gpio_dirin(n)	*(volatile u_int *)((GPIO_OE)) |= 1<<(n)
#define gpio_dirout(n)	*(volatile u_int *)((GPIO_OE)) &= ~(1<<(n))
#define gpio_set(n)		*(volatile u_int *)((GPIO_SETOUT)) = 1<<(n)
#define gpio_clr(n)		*(volatile u_int *)((GPIO_CLROUT)) = 1<<(n)
#define gpio_get(n)		( ( *(volatile u_int *)((GPIO_DATAIN)) & (1<<(n)) ) ? 1 : 0 )


#define get_io(r)			*((volatile u_int *)(TI81XX_L4_SLOW_IO_ADDRESS(r)))
#define set_io(r,v)		*((volatile u_int *)(TI81XX_L4_SLOW_IO_ADDRESS(r))) = (v)
#define and_io(r,v)		*((volatile u_int *)(TI81XX_L4_SLOW_IO_ADDRESS(r))) &= (v)
#define or_io(r,v)			*((volatile u_int *)(TI81XX_L4_SLOW_IO_ADDRESS(r))) |= (v)



#define CPLD_RST_VIN4				(1<<7)
#define CPLD_RST_VIN3				(1<<6)
#define CPLD_RST_VIN2				(1<<5)
#define CPLD_RST_VIN1				(1<<4)


typedef struct{
	u_int	devType;	//0xb0: DS-7216HV-ST board DS-80013 0XB1: DS-8116HC/HS-ST豸ͺDS-80022

	u_short	cpldDate;	//Ver
	u_short cfgbit;	//bit0=0x10en|bit1=0x20en|bit2=0x30en|bit3=0x40en

	u_char	devFreq;	//hi3520 freq: 0x00=600MHz	
	u_char	ddrFreq;	//DDR2 freq:   0x00=400MHz
	u_char	ddrType;	//DDR2 size:   0x00=384MB DDR SDRAM3Ƭ64M X 16 SDRAM01512MB DDR SDRAM2Ƭ64M X 16 SDRAM + 2Ƭ128M X 8 SDRAM
	u_char	flashType;	//0x0=nor|0x1=nand

	u_char  flashCnt;	//0x0=1chip
	u_char  flashSize;	//0x0=16M|0x1=32M
	u_char	ethPort;	//0x0=1port|
	u_char	idePort;	//0x00=0port|0x01=2port

	u_char	viType;	//0x00=TTW2866
	u_char  vgaType;	//0x0=hi3520 intel
	u_char  voType;	//0x0=hi3520 intel
	u_char  hasVga;	//0x0=has

	u_char	phyType;	//0x00=RTL8211BN 0x01=RTL8201BN
	u_char  netType;	//0x0=10/100M|0x1=10/100/1000M
	u_char	cpldType;	//0x0=Altera
	u_char  ideType;	//0x00=sata

	u_char  sataType;	//0x00=none|0x01=sii3114|0x10=atp8624
	u_char  cfg_fpga;	//0x0=none
	u_char  fpgaType;	//0x0=none
	u_char  voPort;	//0x0=1port|0x1=2port

	u_char	usbPort;	//0x0=2port
	u_char	res[35];

} cpld_info;


typedef struct _hdd_exc_
{
	void *dev; /* kernel use, override by magic in ioctl */
	u32 readErr;
	u32 writeErr;
	u32 keyAreaErr; /*lba in key area*/
	u32 lbaOverflow; /*lba out of range*/
	u32 NoReady; /*disk no ready*/
	u32 BSYCount; /*disk busy*/
	u32 DRQSetErr; /*DRQ not set*/
	u32 hdStatus; /* disk status: idle error exist */

	u32 retryCmdCount;
	u32 abortCmdCount; /* host initiative */
	u32 CmdAborted; /* passivity */
	u32 CmdOverUnknown;
	u32 CmdOverErr;

	u32 BadSCSICmdParam; /* scsi comand parameter bad */
	u32 UAReset; /* unit attention bus reset */
	u32 UAParamChanged; /* unit attention parameter changed */
	u32 QueueFull; /* scsi comand queue full */
	u32 OverRun; /* edma reg SerrInt*/
	u32 UnderRun;
	u32 ParityErr;
	u32 BadSCSICmdResult;

	u32 DeviceErr; /* Neither device error without completion nor self disable must be in NCQ
					* mode and Port multiplier connected*/
	u32 RecoverableErr;
	u32 SerrInt; /* one bit set in SError*/
	u32 LinkCtlRxErr;
	u32 LinkCtlTxErr;
	struct timespec diskAccessTime;
	u32 reserved1;
	u32 reserved2;
} hdd_exc;

#endif

