#include <linux/module.h>
#include <linux/version.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/cdev.h>
#include <asm/io.h>
#include <linux/syscalls.h>
#include <linux/mtd/partitions.h>
#include <linux/proc_fs.h>
#include "hikdvlib.h"

#define HIKIO							"hikio"

#define HIKIO_GPIO_SET				0x1
#define HIKIO_GPIO_GET				0x2
#define HIKIO_GPIO_DIR				0x3
#define HIKIO_GET_CPLD_INFO			0x4
#define HIKIO_PINMUX_SWITCH			0x5
#define HIKIO_GET_BOOT_PARMS		0x10
#define HIKIO_GET_DEV_HARD_INFO 	0x11
#define HIKIO_WD_INIT				0x20
#define HIKIO_WD_FEED				0x21
#define HIKIO_SPI_INIT				0x27
#define HIKIO_SPI_READ				0x28
#define HIKIO_SPI_WRITE				0x29
#define HIKIO_GET_DEVFREQ			0x30
#define HIKIO_AD_RESET				0x31
#define HIKIO_VGA_DETECT			0x32
#define HIKIO_FLASH_INFO			0x33
#define HIKIO_GPIO_INFO				0x34
#define HIKIO_GET_CPLD				0x50
#define HIKIO_SET_CPLD				0x51
#define IOCTL_GET_HDD_EXC       	0x081018
#define IOP_GET_STATUS 				0x80026b90
#define IOP_SET_MODE   				0x40026b91

#define CPLD_MODE_00_15			0x22
#define CPLD_READ_00_15			0x22
#define CPLD_INTR_00_15			0x26
#define CPLD_INTW_00_15			0x26
#define CPLD_OUTP_00_07			0x1a

#define GPIO_DIR_OUT			0x0
#define GPIO_DIR_IN				0x1
#define GPIO_LOW				0x0
#define GPIO_HIGH				0x1

static u_int GPIO_BASE = GPIO0_BASE;

#define GPIO_BEEP			20

#define RAMDISK_MAX_SIZE		(CONFIG_BLK_DEV_RAM_SIZE*SZ_1K)

static u_char g_ori_cpldinfo[256];
static u_char g_boot_params[256];
static int g_dbg = 0;
static u_char *g_cpld_base = NULL;

int g_feeddog = 1;
#ifdef CONFIG_MTD_NAND
int g_is_evm = 0; //nand flash
#else
int g_is_evm = 0; //nor flash
#endif
int g_sata_pmp = 1;
int g_rs232_mode = 0;

u_int g_sensor_type;/* use for alarm */

EXPORT_SYMBOL(g_dbg);
static int __init dbg_switch(char *str)
{
	if(str) {
		g_dbg = str[0]-'0';
	} else {
		g_dbg = 0;
	}
	console_printk[0] = g_dbg;
	return 0;
}
early_param("dbg",dbg_switch);

EXPORT_SYMBOL(g_is_evm);
static int __init evm_switch(char *str)
{
	if(str) {
		g_is_evm = str[0]-'0';
	} else {
		g_is_evm = 0;
	}
	return 0;
}
early_param("evm",evm_switch);

EXPORT_SYMBOL(g_sata_pmp);
static int __init sata_pmp_switch(char *str)
{
	if(str) {
		g_sata_pmp = str[0]-'0';
	} else {
		g_sata_pmp = 0;
	}
	return 0;
}
early_param("sata-pmp",sata_pmp_switch);

static u_int mk_kernel_build_date(void)
{
	const int YEAR_SHIFT = 16;
	const int YEAR_MASK  = 0xffff;
	const int MON_SHIFT	 = 8;
	const int MON_MASK	 = 0xff;
	const int DAY_SHIFT	 = 0;
	const int DAY_MASK	 = 0xff;

	int		year, month, day;
	char	m[4];
	char	*months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

	sscanf(__DATE__, "%s%d%d", m, &day, &year);

	for(month=0; month<12; month++)
		if(strcmp(m, months[month]) == 0)
			break;

	month++;
	year -= 2000;

	return (unsigned int) (((year&YEAR_MASK)<<YEAR_SHIFT) |
						   ((month&MON_MASK)<<MON_SHIFT) |
						   ((day&DAY_MASK)<<DAY_SHIFT));
}

static int hikio_open(struct inode *inode, struct file *filp)
{
	try_module_get(THIS_MODULE);
	return 0;
}

static int hikio_close(struct inode *inode, struct file *filp)
{
	module_put(THIS_MODULE);
	return 0;
}	

static u_short g_alarm_in_00_15 = 0;

static int hikio_ioctl(struct file *filp, u_int cmd, u_long arg)
{
	u_long param;

	param = arg;
	
	switch(cmd)
	{	
		case HIKIO_GPIO_SET:
			if((param&0xff)>=32) {
				GPIO_BASE = GPIO1_BASE;
				param -= 32;
			}
			if( ((param>>8) & 0x1) == GPIO_HIGH ) {
				gpio_set(param&0xff);
			} else {
				gpio_clr(param&0xff);
			}
			break;
		case HIKIO_GPIO_GET:
			if((param&0xff)>=32) {
				GPIO_BASE = GPIO1_BASE;
				param -= 32;
			}			
			return gpio_get(param&0xff);/* duhao fix */
		case HIKIO_GPIO_DIR:
			if((param&0xff)>=32) {
				GPIO_BASE = GPIO1_BASE;
				param -= 32;
			}			
			if( ((param>>8) & 0x1) == GPIO_DIR_IN ) {
				gpio_dirin(param&0xff);
			} else {
				gpio_dirout(param&0xff);
			}
			break;			
		case HIKIO_GET_CPLD_INFO:
		{
			copy_to_user((void*)arg, (void*)&g_ori_cpldinfo, sizeof(g_ori_cpldinfo));
			break;			
		}
		case HIKIO_GET_BOOT_PARMS:
		{
			copy_to_user((void *)(arg), &g_boot_params, sizeof(g_boot_params));
			break;
		}
		case HIKIO_GET_DEV_HARD_INFO:
		{
			u_int kver, kdate;
			if(!arg) {
				return -EINVAL;
			}
			kver = LINUX_VERSION_CODE;/* kernelsoftwareVersion */
			kdate = mk_kernel_build_date();
			copy_to_user((void *)(arg+2*sizeof(u_int)), &kver, sizeof(u_int));
			copy_to_user((void *)(arg+3*sizeof(u_int)), &kdate, sizeof(u_int));		
			break;
		}		

		case IOCTL_GET_HDD_EXC:
		{
			extern void hik_get_hdd_exc(int idx,void *arg);
			
			u32 idx = *(u32 *)arg;
			
			if (idx >= 8) {/* MAX_DISK */
				printk("disk index[%d] < max disk count[%d]\n",idx, 8);
				return -EINVAL;
			}

			hik_get_hdd_exc(idx,arg);

			*(u32 *)arg = IOCTL_GET_HDD_EXC; /* take it as magic */
			break;
		}
		
		case IOP_GET_STATUS:
		{
			u_int alarm_in;
			u_short alarm_in_00_15;
			u_short alarm_in_16_32;

			alarm_in_00_15 = *(volatile u_short *)(g_cpld_base + CPLD_READ_00_15);
			alarm_in_16_32 = 0;//*(volatile u_short *)(g_cpld_base + CPLD_READ_16_32);

			alarm_in = (alarm_in_16_32<<16) | alarm_in_00_15;
			alarm_in = alarm_in ^ g_sensor_type;

			copy_to_user((void *)arg,&alarm_in,sizeof(u_int));
			
			break;
		}
		case IOP_SET_MODE:
		{	
			*(volatile u_short *)(g_cpld_base + CPLD_MODE_00_15) = 0;/* always open */
			g_sensor_type = arg;
			break;		
		}

		default:
			return -EINVAL;		
	}


	return 0;
}

ssize_t hikio_write(struct file *filp, const char __user *buf, size_t size, loff_t *pos)
{
	u_int alarm_out = 0;
	if(size > 4 || buf == NULL) 
	{
		return -EINVAL;
	}
	copy_from_user(&alarm_out,buf,size);
	*(volatile u_char *)(g_cpld_base + CPLD_OUTP_00_07) = alarm_out & 0xff;

	return size;
}

static struct file_operations hikio_fops = {
	open:		hikio_open,	
	release:	hikio_close,
	unlocked_ioctl:		hikio_ioctl,
	write:		hikio_write,	
	
};
static int g_hikio_major;

static irqreturn_t gpio0_isr(int irq, void *dev)
{
	u_short regval;

	u_int gpio_int_status;
	gpio_int_status = get_io(0x48032000+0x2c);/* GPIO_IRQSTATUS_0 */

	if(gpio_int_status & 0x1) 
	{
		regval = *(volatile u_short *)(g_cpld_base + CPLD_INTR_00_15);
		//g_alarm_in_00_15 = *(volatile u_short *)(g_cpld_base + CPLD_READ_00_15);
		//printk("gpio0_isr,cpld_intr=%x,alarm_in=%x\n",regval,g_alarm_in_00_15);

		*(volatile u_short *)(g_cpld_base + CPLD_INTW_00_15) = regval;/* clear cpld intrrupt */
		*(volatile u_short *)(g_cpld_base + CPLD_INTW_00_15) = 0;/* re enable cpld intrrupt */
	} else {
		printk("gpio0 intrrupt,%x\n",gpio_int_status);
	}
	set_io(0x48032000 + 0x2c,gpio_int_status);/* clear gpio0 status */
	
	return IRQ_HANDLED;
}

void beep_on_m100(void)
{
	gpio_dirout(GPIO_BEEP);
	gpio_clr(GPIO_BEEP);
	mdelay(100);
	gpio_set(GPIO_BEEP);
}
EXPORT_SYMBOL(beep_on_m100);
void hik_board_init(void)
{
	
	//extern void load_cpld(void);
	beep_on_m100();

#if 0
	/* alarm in intrrupt gp0[0] */
	gpio_dirin(0);
	set_io(0x48032000+0x34,1);/* enable intrrupt */
	set_io(0x48032000+0x14c,1);/* set fail edge */
	if(1)
	{
		u_int gpio = 0;
		int ret = request_irq(96, gpio0_isr, IRQF_SHARED, "gpio0", (void *)&gpio);
	}
#endif	
	/* set pinmux add caohongkong 2011.03.23*/
	unsigned int tmp;
	tmp=*(unsigned int*)TI81XX_L4_SLOW_IO_ADDRESS(0x4814092c);
	tmp=((tmp&0xfffcff00)|0x04);
	set_io(0x4814092c,tmp);		/*	UART1	TXD	*/
	tmp=*(unsigned int*)TI81XX_L4_SLOW_IO_ADDRESS(0x48140930);
	tmp=((tmp&0xfffcff00)|0x04);
	set_io(0x48140930,tmp);		/*	UART1 	RXD	*/

	tmp=*(unsigned int*)TI81XX_L4_SLOW_IO_ADDRESS(0x4814090c);
	tmp=((tmp&0xfffcff00)|0x80);
	set_io(0x4814090c,tmp);	/*	RE_485#GP1_0	*/
	tmp=*(unsigned int*)TI81XX_L4_SLOW_IO_ADDRESS(0x48140910);
	tmp=((tmp&0xfffcff00)|0x80);
	set_io(0x48140910,tmp);	/*	TE_485#GP1_1	*/
	//set_io(0x48140b00,1);/* video input intr GP1_13 */
	//set_io(0x48140b30,2);/* en_VGA_i2c GP0_30 */

	tmp=*(unsigned int*)TI81XX_L4_SLOW_IO_ADDRESS(0x48140854);
	tmp=((tmp&0xfffcff00)|0x20);
	set_io(0x4814090c,tmp);	/*	I2C[3]_SCL	*/
	tmp=*(unsigned int*)TI81XX_L4_SLOW_IO_ADDRESS(0x48140858);
	tmp=((tmp&0xfffcff00)|0x20);
	set_io(0x48140910,tmp);	/*	I2C[3]_SDA	*/
	
	
	/* enable cs2 use for cpld */
	//*(volatile u_int *)TI81XX_GPMC_IO_ADDRESS(0x500000d8) = 0xf44;
}


static int __init hikgpio_init(void)
{	
	extern void init_key_area(void);

#if 0	
	init_key_area();
	/*-------- load boot_params for flash ---------------*/
	{
		#include <linux/mtd/partitions.h>
		memset(g_boot_params,0,sizeof(g_boot_params));
		{
			extern struct mtd_partition ti8148_evm_norflash_partitions[];
			u_int bp_off = ti8148_evm_norflash_partitions[0].size+0x8008000;
			u_int *bp_virt = (u_int *)ioremap(bp_off,0x8000);
			if(bp_virt) {/* map boot_params address in flash ok */
				if(*bp_virt == 0x484b5753) {/* check magic "HKWS" */
					memcpy(g_boot_params,(void *)bp_virt,sizeof(g_boot_params));
					printk("load boot params from flash[%x] ok.\n",bp_off);				
				}		
				iounmap((void*)bp_virt);			
			}
		}
	}
#endif	
	g_hikio_major = register_chrdev(0, HIKIO, &hikio_fops);
	                                                                          
	if (g_hikio_major < 0) 
	{
        printk("hikio: failed to allocate major number.\n");
	    return -ENODEV;
    }
	printk("Successfully initialized hikio module\n");

	hik_board_init();

#if 0
	extern int hikvir_init(void);
	extern char DesData[256];
	hikvir_init();

	if(memcmp(DesData, g_boot_params, 256) != 0) {	
		g_feeddog = 0;
	}


	//g_cpld_base = (u_char *)ioremap(0x04000000,0x100);
	//memcpy(g_ori_cpldinfo,g_cpld_base,0x100);

#endif
	return 0;
}

static void __exit	hikgpio_exit(void)
{
	iounmap(g_cpld_base);
	unregister_chrdev(g_hikio_major, HIKIO);
	
	return;
}

module_init(hikgpio_init);
module_exit(hikgpio_exit);

#define RTFS_BUF_SZ	(PAGE_SIZE*16)
static void load_rootfs(u_char *membase)
{
#ifdef CONFIG_BLK_DEV_INITRD
	//extern u_int g_memsize;
	extern u_long initrd_start;
	int fdin,fdout,size = 0;
	u_char *buf = membase;

	if(!initrd_start) {/* boot from nfs */
		return;
	}

	//printk("----------->%x\n",membase);
	printk("loading rootfs ... ");
	if(buf) {	
		
		fdin = sys_open("/root/rootfs.img",O_RDONLY,0777);
		if(fdin != -1) {
			int len = sys_lseek(fdin,0,2);
			fdout = sys_open("/initrd.image", O_WRONLY|O_CREAT, 0777);
			sys_lseek(fdin,0,0);


			if(len>0) {
				while(1) {
					size = sys_read(fdin,buf,RTFS_BUF_SZ);
					if(size>0) {		
						int ret = sys_write(fdout,buf,size);
						if(ret != size) {
							printk("write rootfs failed.%d\n",ret);
							break;
						}
					}
					if( size < RTFS_BUF_SZ ) {
						break;
					}
				}
			}
			sys_close(fdin);
			sys_close(fdout);
			printk("done.\n");
		} else {
			printk("failed.%d\n",size);
		}
	}
#endif	
}


void load_jffs2_file(void)/* duhao add */
{

	//extern u_int g_memsize;
	int ret,fs_dev_idx;
	u_char *base;
	u_char mtdname[64];
	u_char fstype[20];

	extern int g_is_evm;
	if(g_is_evm)
		return;

#ifdef CONFIG_HIK_ENC
	extern u_int InterOperate1, InterOperate2, InterOperate3;
	if( (InterOperate1 != 0x1) || (InterOperate2 != 0x0) )
		return ;
#endif

#ifdef CONFIG_MTD_NAND
	fs_dev_idx = 3;
	strcpy(fstype,"yaffs");
#else
	fs_dev_idx = 2;
	strcpy(fstype,"jffs2");	
#endif	
	//base = (u_char *)ioremap(PHYS_OFFSET + g_memsize,RAMDISK_MAX_SIZE);
	base = (u_char *)kmalloc(RTFS_BUF_SZ,GFP_KERNEL);
	sprintf(mtdname,"/dev/mtdblock%d",fs_dev_idx);
	sys_mknod(mtdname,S_IFBLK|00777,fs_dev_idx|(31<<8));
	ret = sys_mount(mtdname, "/root", fstype, MS_RDONLY, NULL);/* duhao add */
	if(ret>=0) {
		int fd;
		
		printk("mount filesystem ok.\n");

		/*--------------check console -----------------*/
		fd = sys_open("/root/USAGE232",O_RDONLY,00777);
		if(fd>=0)  {
			u_char console_type[20];
			int len = sys_lseek(fd,0,2);
			sys_lseek(fd,0,0);
			ret = sys_read(fd, console_type, len);
			if(ret == len)  {
				if( memcmp(console_type, "TRANSMISSION", len) == 0 ) {
					g_rs232_mode = 1;
				}
			}
		}
		sys_close(fd);
	
		/*--------------load rootfs-----------------*/
		load_rootfs(base);
		sys_umount("/root",MNT_DETACH);
	} else {
		printk("mount filesystem failed,%d\n",ret);
	}
	kfree(base);
	//iounmap((void *)base);	
}

