#include <linux/i2c.h>
#include <linux/bcd.h>
#include <linux/rtc.h>
#include <linux/slab.h>
#include <asm/delay.h>
#if 1
/* pfc8563 register offsets */
#define CFG_I2C_RTC_ADDR            (0xa2 >> 1) 
#define RTC_REGISTER_CTRL1          0x00
#define RTC_REGISTER_CTRL2          0x01
#define RTC_REGISTER_SEC            0x02
#define RTC_REGISTER_MIN            0x03
#define RTC_REGISTER_HOUR           0x04
#define RTC_REGISTER_DAY            0x05
#define RTC_REGISTER_WDAY           0x06
#define RTC_REGISTER_MONTH          0x07
#define RTC_REGISTER_YEAR           0x08
#define RTC_REGISTER_ALARM_MIN      0x09
#define RTC_REGISTER_ALARM_HOUR     0x0a
#define RTC_REGISTER_ALARM_DAY      0x0b
#define RTC_REGISTER_ALARM_WDAY     0x0c
#define RTC_REGISTER_CLKOUT         0x0d
#define RTC_REGISTER_TIMERCTRL      0x0e
#define RTC_REGISTER_TIMER          0x0f

static struct i2c_client *g_rtc_client = NULL;


static int rtc_i2c_read_reg(u8 reg, u8 * val)
{
	unsigned char buf[4];
	struct i2c_msg msgs[2];

	buf[0] = reg;
	
	msgs[0].addr = g_rtc_client->addr;
	msgs[0].flags = 0;
	msgs[0].len = 1;
	msgs[0].buf = buf;

	msgs[1].addr = g_rtc_client->addr;
	msgs[1].flags = I2C_M_RD;
	msgs[1].len = 1;
	msgs[1].buf = buf;

	/* read registers */
	if ((i2c_transfer(g_rtc_client->adapter, msgs, 2)) != 2) {
		dev_err(&g_rtc_client->dev, "%s: read error\n", __func__);
		return -EIO;
	}
	
	*val = buf[0];
	return 0;
}


static int rtc_i2c_write_reg(u8 reg, u8 val)
{
	u_char data[2] = { reg, val };
	int ret;
	ret = i2c_master_send(g_rtc_client, data, sizeof(data));
	if (ret != sizeof(data)) {
		dev_err(&g_rtc_client->dev,"%s: err=%d addr=%02x, data=%02x\n",__func__, ret, data[0], data[1]);
		return -EIO;
	}
	return 0;
}

static int rtc8563_read(unsigned char adr, unsigned char *buf, unsigned char len)
{
	int i;
	for(i=0;i<len;i++) {
		rtc_i2c_read_reg(adr,buf);
		adr++;
		buf++;
	}
	return 0;

}

static int rtc8563_write(unsigned char adr, unsigned char *data, unsigned char len)
{
	int i;
	for(i=0;i<len;i++) {	
		rtc_i2c_write_reg(adr, *data);
		adr++;
		data++;
	}
	return 0;
}



static int hik_rtc_read_time(struct device *dev, struct rtc_time *tm)
{
	char tmpMon;
	char tmpYear;
	unsigned char buf[7];

#ifdef CONFIG_HIK_ENC
	extern u_int InterOperate1, InterOperate2, InterOperate3;
	static int check_count = 0;
	if( (InterOperate2 != 0x0) || (InterOperate3 != 0x5a) || (InterOperate1 != 0x1) ) {
		check_count++;
		if(check_count>5) {
			while(1);/* dead loop */
		}
	}
#endif		
	/*
 	 * yxq fixed at 20080105 for get rid of the time carry then it will get
	 * a error result, so must write the value a one operation
 	 */
	memset(buf, 0, sizeof(buf));
	rtc8563_read(RTC_REGISTER_SEC, buf, 7);
	tmpMon = buf[5];
	tmpYear = buf[6];
	if(tmpMon >= 0x80) {
		tm->tm_year = bin2bcd (tmpYear) + 2000 - 1900;
	} else {
		tm->tm_year = bin2bcd (tmpYear);   /* + 1900 - 1900; */
	}
	tmpMon &= 0x1f;
	tm->tm_mon = bin2bcd (tmpMon) - 1;

	// get the mday wday hour, min, sec
	buf[0]&=0x7f;
	tm->tm_sec = bin2bcd(buf[0]);
	buf[1]&=0x7f;
	tm->tm_min = bin2bcd(buf[1]);
	buf[2]&=0x3f;
	tm->tm_hour = bin2bcd(buf[2]);
	buf[3]&=0x3f;
	tm->tm_mday = bin2bcd(buf[3]);
	buf[4]&=0x7;
	tm->tm_wday = bin2bcd(buf[4]);
	
	//printk("get the reg, org is %d-%d-%d  %d:%d:%d!\n", tm->tm_year, tm->tm_mon, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);

	return 0;
}

static int hik_rtc_set_time(struct device *dev, struct rtc_time *tm)
{
	struct timespec tv;
	unsigned char year, wday, mon, day, hrs, min, sec;
	unsigned int yrs, cFlag=0;
	unsigned char buf[7];

	yrs = tm->tm_year + 1900;	
	mon = tm->tm_mon + 1;
	wday = tm->tm_wday;
	hrs = tm->tm_hour;
	day = tm->tm_mday;
	min = tm->tm_min;
	sec = tm->tm_sec;
	if (yrs < 1970 || yrs > 2037)
		return -EINVAL;
	if ((mon > 12) || (day == 0))
		return -EINVAL;
	if ((hrs >= 24) || (min >= 60) || (sec >= 60))
		return -EINVAL;

	if(yrs > 2000)
	{
		yrs -= 2000;
		cFlag = 0x80;
	}else{
		yrs -= 1900;
	}
	year = (char)yrs;
	//printk("set the reg, org is %d-%d-%d  %d:%d:%d!\n", yrs, mon, day, hrs, min, sec);

	/*
 	 * yxq fixed at 20080105 for get rid of the time carry then it will get
	 * a error result, so must write the value a one operation
 	 */
	memset(buf, 0, sizeof(buf));
	buf[0] =  bin2bcd(sec);
	buf[1] =  bin2bcd(min);
	buf[2] =  bin2bcd(hrs);
	buf[3] =  bin2bcd(day);
	buf[4] =  bin2bcd(wday);
	buf[5] =  bin2bcd(mon) | cFlag;
	buf[6] =  bin2bcd(year);
	rtc8563_write(RTC_REGISTER_SEC, buf, 7);

	tv.tv_nsec = 0;
	tv.tv_sec = mktime (tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
		tm->tm_hour, tm->tm_min, tm->tm_sec+2);
	do_settimeofday (&tv);
	
	return 0;
}


static const struct rtc_class_ops davinci_rtc_ops = {
	.read_time	= hik_rtc_read_time,
	.set_time	= hik_rtc_set_time,
};

static struct i2c_driver pcf8563_driver;

struct pcf8563 {
	struct rtc_device *rtc;
	/*
	 * The meaning of MO_C bit varies by the chip type.
	 * From PCF8563 datasheet: this bit is toggled when the years
	 * register overflows from 99 to 00
	 *   0 indicates the century is 20xx
	 *   1 indicates the century is 19xx
	 * From RTC8564 datasheet: this bit indicates change of
	 * century. When the year digit data overflows from 99 to 00,
	 * this bit is set. By presetting it to 0 while still in the
	 * 20th century, it will be set in year 2000, ...
	 * There seems no reliable way to know how the system use this
	 * bit.  So let's do it heuristically, assuming we are live in
	 * 1970...2069.
	 */
	int c_polarity;	/* 0: MO_C=1 means 19xx, otherwise MO_C=1 means 20xx */
};

static int pcf8563_probe(struct i2c_client *client, const struct i2c_device_id *id)
{
	struct pcf8563 *pcf8563;

	int err = 0;

	dev_dbg(&client->dev, "%s\n", __func__);

	if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C))
		return -ENODEV;
	client->addr = CFG_I2C_RTC_ADDR;
	g_rtc_client = client;
	
#if 1
	pcf8563 = kzalloc(sizeof(struct pcf8563), GFP_KERNEL);
	if (!pcf8563)
		return -ENOMEM;

	i2c_set_clientdata(client, pcf8563);
	
	pcf8563->rtc = rtc_device_register(pcf8563_driver.driver.name,
				&client->dev, &davinci_rtc_ops, THIS_MODULE);

	if (IS_ERR(pcf8563->rtc)) {
		err = PTR_ERR(pcf8563->rtc);
		goto exit_kfree;
	}
	
	return 0;

exit_kfree:
	kfree(pcf8563);
#endif

	return err;
}

static int pcf8563_remove(struct i2c_client *client)
{
#if 1
	struct pcf8563 *pcf8563 = i2c_get_clientdata(client);

	if (pcf8563->rtc)
		rtc_device_unregister(pcf8563->rtc);

	kfree(pcf8563);
#endif
	return 0;
}
static const struct i2c_device_id pcf8563_id[] = {
	{ "pcf8563", 0 },
	{ "rtc8564", 0 },
	{ }
};
MODULE_DEVICE_TABLE(i2c, pcf8563_id);

static struct i2c_driver pcf8563_driver = {
	.driver		= {
		.name	= "rtc-pcf8563",
	},
	.probe		= pcf8563_probe,
	.remove		= pcf8563_remove,
	.id_table	= pcf8563_id,
};

static int init_hik_rtc(void)
{
	int ret;
	char tmp;

	i2c_add_driver(&pcf8563_driver);
	printk("<duhao>init hik rtc\n");

	ret = rtc_i2c_read_reg (RTC_REGISTER_SEC, &tmp);
	if(ret<0)
	{
		if(ret == -EREMOTEIO) {
			printk("rtc: hardware not connected\n");
		} else {
			printk("<7>rtc: init_hik_rtc failed,err=%d\n",ret);
		}
		return -1;
	}
	
	/* clear all control & status registers */
	ret = rtc_i2c_write_reg (RTC_REGISTER_CTRL1, 0x00);
	ret = rtc_i2c_write_reg (RTC_REGISTER_CTRL2, 0x00);
	ret = rtc_i2c_write_reg (RTC_REGISTER_CLKOUT, 0x00);

	/* clear Voltage Low bit */
	tmp &= 0x7f;
	ret = rtc_i2c_write_reg (RTC_REGISTER_SEC, tmp);

	/* reset all alarms */
	ret = rtc_i2c_write_reg (RTC_REGISTER_ALARM_MIN, 0x00);
	ret = rtc_i2c_write_reg (RTC_REGISTER_ALARM_HOUR, 0x00);
	ret = rtc_i2c_write_reg (RTC_REGISTER_ALARM_DAY, 0x00);
	ret = rtc_i2c_write_reg (RTC_REGISTER_ALARM_WDAY, 0x00);
	udelay (1000);
	/* set clk out 32hz*/
	ret = rtc_i2c_write_reg (RTC_REGISTER_CLKOUT, 0x82);
	udelay (1000);  

    return 0;
}



struct pcf8563_limit {
	unsigned char reg;	
	unsigned char mask;
	unsigned char min;	
	unsigned char max;
};

/************************************************************************
 *
 * FUNC : check_rtc_validate()
 * DESC : check rtc time validate
 * PARA : none
 *   
 *  RET : 1=valid,0=invalid 
 *
 ************************************************************************/
static int check_rtc_validate(void)
{
	int i,yrs;
	u_char year,month;
	static const struct pcf8563_limit pattern[] = {
		/* register         mask  min  max */
		{ RTC_REGISTER_SEC,	 0x7F,  0, 59 },
		{ RTC_REGISTER_MIN,	 0x7F,  0, 59 },
		{ RTC_REGISTER_HOUR, 0x3F,  0, 23 },
		{ RTC_REGISTER_DAY,	 0x3F,  1, 31 },
		{ RTC_REGISTER_MONTH,0x1F,  1, 12 },
	};

	/* check limits (only registers with bcd values) */
	for (i = 0; i < ARRAY_SIZE(pattern); i++) {
		u_char raw, val;
		raw = val = 0xff;
		rtc_i2c_read_reg (pattern[i].reg, &raw);
		val = bin2bcd(raw & pattern[i].mask);

		if (val > pattern[i].max || val < pattern[i].min) {
			printk("reg=%x, mask=0x%02x, min=%d, max=%d, value=%d, raw=0x%02X\n",
				pattern[i].reg, pattern[i].mask, pattern[i].min, pattern[i].max, val, raw);
			return 0;
		}
	}
	
	rtc_i2c_read_reg(RTC_REGISTER_YEAR,&year);
	rtc_i2c_read_reg(RTC_REGISTER_MONTH,&month);
	if(month & 0x80) {
		yrs = year + 2000;
	} else {
		yrs = year + 1900;
	}
	if(yrs<1970 || yrs>2037) {
		return 0;
	}
	return 1;
}
static int __init davinci_rtc_init(void)
{
	struct timespec tv;
	struct rtc_time tm;

	if( init_hik_rtc()==-1) {
		return -ENODEV;
	}

#if 1
	/* check rtc validate,if invalid reset rtc time */
	if( !check_rtc_validate() ) {
		memset(&tm,0,sizeof(tm));
		tm.tm_year = 1970-1900;
		tm.tm_mday = 1;
		tm.tm_wday = 4;
		hik_rtc_set_time (NULL,&tm);
	} else {
		hik_rtc_read_time (NULL,&tm);
	}
#else
	hik_rtc_read_time (&tm);
#endif
	printk("<0>rtc: [%04d-%02d-%02d %02d:%02d:%02d],week is %d\n", 
		tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday, 
		tm.tm_hour, tm.tm_min, tm.tm_sec, tm.tm_wday);
	
	tv.tv_nsec = 0;
	tv.tv_sec = mktime (tm.tm_year+1900,tm.tm_mon+1,tm.tm_mday, 
		tm.tm_hour,tm.tm_min, tm.tm_sec);
	do_settimeofday (&tv);
	
    //register_rtc (&davinci_rtc_ops);

	return 0;
}

module_init(davinci_rtc_init);
#endif

