/*
 * TI814X Mux data.
 *
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/
 *
 * Derived from: arch/arm/mach-omap2/mux34xx.c Original copyright follows:
 *
 * Copyright (C) 2009 Nokia
 * Copyright (C) 2009 Texas Instruments
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/init.h>

#include "mux.h"

#ifdef CONFIG_OMAP_MUX

#define _TI814X_MUXENTRY(M0, g, m0, m1, m2, m3, m4, m5, m6, m7)		\
{									\
	.reg_offset	= (TI814X_CONTROL_PADCONF_##M0##_OFFSET),	\
	.gpio		= (g),						\
	.muxnames	= { m0, m1, m2, m3, m4, m5, m6, m7 },		\
}

/*
 * Superset of all mux modes for TI814X
 */
static struct omap_mux __initdata ti814x_muxmodes[] = {
	_TI814X_MUXENTRY(MMC1_CLK, 0,
			"mmc1_clk", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MMC1_CMD, 0,
			"mmc1_cmd_mux0", NULL, NULL,
			NULL, NULL, NULL, NULL, "gpio0_0"),
	_TI814X_MUXENTRY(MMC1_DAT0, 0,
			"mmc1_dat0", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MMC1_DAT1, 0,
			"mmc1_dat1", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MMC1_DAT2, 0,
			"mmc1_dat2", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MMC1_DAT3, 0,
			"mmc1_dat3", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(OSC_WAKE, 0,
			"osc_wake", "spi1_cs1", NULL, NULL,
			NULL, NULL, "timer5_mux1", "gpio1_07_mux0"),
	_TI814X_MUXENTRY(MMC0_CLK, 0,
			"mmc0_clk", NULL, NULL, "sc1_c4",
			NULL, NULL, NULL, "gpio0_1"),
	_TI814X_MUXENTRY(MMC0_CMD, 0,
			"mmc0_cmd", "mmc1_cmd_mux1", NULL, "sc1_det",
			NULL, NULL, NULL, "gpio0_2"),
	_TI814X_MUXENTRY(MMC0_DAT0, 0,
			"mmc0_dat0", "mmc1_dat4", NULL, "sc1_data",
			NULL, NULL, NULL, "gpio0_3"),
	_TI814X_MUXENTRY(MMC0_DAT1, 0,
			"mmc0_dat1", "mmc1_dat5", NULL, "sc1_clk",
			NULL, NULL, NULL, "gpio0_4"),
	_TI814X_MUXENTRY(MMC0_DAT2, 0,
			"mmc0_dat2", "mmc1_dat6", NULL, "sc1_rst",
			NULL, NULL, NULL, "gpio0_5"),
	_TI814X_MUXENTRY(MMC0_DAT3, 0,
			"mmc0_dat3", "mmc1_dat7", NULL, "sc1_vccen",
			NULL, NULL, NULL, "gpio0_6"),
	_TI814X_MUXENTRY(XREF_CLK0, 0,
			"xref_clk0", "mcasp0_axr_7_mux1", "mcasp0_ahclkx",
			"mcasp3_ahclkx", "atlclkout1", "atlclkout0", "vic0",
			"usb1_drvvbus"),
	_TI814X_MUXENTRY(XREF_CLK1, 0,
			"xref_clk1", "mcasp0_axr_8_mux1", "mcasp1_ahclkx",
			"mcasp4_ahclkx", "atlclkout2", "xdma_evt_3_mux1",
			"timer2_mux1", "gpio0_8"),
	_TI814X_MUXENTRY(XREF_CLK2, 0,
			"xref_clk2", "mcasp0_axr_9_mux1", "mcasp2_ahclkx",
			"mcasp5_ahclkx", "atlclkout3", "xdma_evt_2_mux1",
			"timer3_mux1", "gpio0_9"),
	_TI814X_MUXENTRY(MCASP0_ACLKX, 0,
			"mcasp0_aclkx", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_FSX, 0,
			"mcasp0_fsx", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_CLKR, 0,
			"mcasp0_aclkr", "mcasp5_axr2", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_FSR, 0,
			"mcasp0_fsr", "mcasp5_axr3", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR0, 0,
			"mcasp0_axr0", NULL, NULL, NULL,
			NULL, "vic1", NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR1, 0,
			"mcasp0_axr1", NULL, NULL, NULL,
			NULL, "i2c3_scl_mux0", NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR2, 0,
			"mcasp0_axr2", NULL, NULL, NULL,
			NULL, "i2c3_sda_mux0", NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR3, 0,
			"mcasp0_axr3", NULL, NULL, "sc1_vppen",
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR4, 0,
			"mcasp0_axr4", "mcasp1_axr_8_mux0", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR5, 0,
			"mcasp0_axr5", "mcasp1_axr_9_mux0", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR6, 0,
			"mcasp0_axr6", "mcbsp_dr", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR7, 0,
			"mcasp0_axr_7_mux0", "mcbsp_dx", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR8, 0,
			"mcasp0_axr_8_mux0", "mcbsp_fsx", "mcbsp_fsr_mux1",
			NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP0_AXR9, 0,
			"mcasp0_axr_9_mux0", "mcbsp_clkx", "mcbsp_clkr_mux1",
			NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_ACLKX, 0,
			"mcasp1_aclkx", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_FSX, 0,
			"mcasp1_fsx", NULL, NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_CLKR, 0,
			"mcasp1_aclkr", "mcasp1_axr4", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_FSR, 0,
			"mcasp1_fsr", "mcasp1_axr5", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_AXR0, 0,
			"mcasp1_axr0", "mmc0_dat4", NULL, "sc0_rst",
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_AXR1, 0,
			"mcasp1_axr1", "mmc0_dat5", NULL, "sc0_det",
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_AXR2, 0,
			"mcasp1_axr2", "mcbsp_fsr_mux0", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP1_AXR3, 0,
			"mcasp1_axr3", "mcbsp_clkr_mux0", NULL, NULL,
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP2_CLKX, 0,
			"mcasp2_aclkx", NULL, NULL, "sc0_c4",
			NULL, NULL, NULL, "gpio0_10_mux1"),
	_TI814X_MUXENTRY(MCASP2_FSX, 0,
			"mcasp2_fsx", NULL, NULL, "sc0_data",
			NULL, NULL, NULL, "gpio0_11_mux1"),
	_TI814X_MUXENTRY(MCASP2_AXR0, 0,
			"mcasp2_axr0", "mmc0_dat6", NULL, "sc0_clk", NULL,
			"uart5_rxd_mux3", NULL, "gpio0_12_mux1"),
	_TI814X_MUXENTRY(MCASP2_AXR1, 0,
			"mcasp2_axr1", "mmc0_dat7", NULL, "sc0_vccen", NULL,
			"uart5_txd_mux3", NULL, "gpio0_13_mux1"),
	_TI814X_MUXENTRY(MCASP2_AXR2, 0,
			"mcasp2_axr2", "mcasp1_axr6", NULL, "sc0_vppen", NULL,
			NULL, "timer2_mux0", "gpio0_14_mux1"),
	_TI814X_MUXENTRY(MCASP2_AXR3, 0,
			"mcasp2_axr3", "mcasp1_axr7", NULL, "tso0_pacerr", NULL,
			NULL, "timer3_mux0", "gpio0_15_mux1"),
	_TI814X_MUXENTRY(MCASP3_CLKX, 0,
			"mcasp3_aclkx", NULL, NULL, "tsi0_dclk",
			NULL, NULL, NULL, "gpio0_16_mux1"),
	_TI814X_MUXENTRY(MCASP3_FSX, 0,
			"mcasp3_fsx", NULL, NULL, "tsi0_bytstrt",
			NULL, NULL, NULL, "gpio0_17_mux1"),
	_TI814X_MUXENTRY(MCASP3_AXR0, 0,
			"mcasp3_axr0", NULL, NULL, "tsi0_data",
			NULL, NULL, "timer4_mux0", "gpio0_18_mux1"),
	_TI814X_MUXENTRY(MCASP3_AXR1, 0,
			"mcasp3_axr1", NULL, NULL, "tsi0_pacval",
			NULL, NULL, "timer5_mux0", "gpio0_19_mux1"),
	_TI814X_MUXENTRY(MCASP3_AXR2, 0,
			"mcasp3_axr2", "mcasp1_axr_8_mux1", NULL, "tsi0_pacerr",
			NULL, NULL, NULL, "gpio0_20_mux1"),
	_TI814X_MUXENTRY(MCASP3_AXR3, 0,
			"mcasp3_axr3", "mcasp1_axr_9_mux1", NULL, "tsi2_pacerr",
			NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MCASP4_CLKX, 0,
			"mcasp4_aclkx", NULL, NULL, "tsi2_dclk",
			NULL, NULL, NULL, "gpio0_21_mux1"),
	_TI814X_MUXENTRY(MCASP4_FSX, 0,
			"mcasp4_fsx", NULL, NULL, "tsi2_bytstrt",
			NULL, NULL, NULL, "gpio0_22_mux1"),
	_TI814X_MUXENTRY(MCASP4_AXR0, 0,
			"mcasp4_axr0", NULL, NULL, "tsi2_data",
			NULL, NULL, NULL, "gpio0_23_mux1"),
	_TI814X_MUXENTRY(MCASP4_AXR1, 0,
			"mcasp4_axr1", NULL, NULL, "tsi2_pacval",
			NULL, NULL, "timer6_mux0", "gpio0_24_mux1"),
	_TI814X_MUXENTRY(MCASP5_CLKX, 0,
			"mcasp5_aclkx", NULL, NULL, "tso0_dclk",
			NULL, NULL, NULL, "gpio0_25_mux1"),
	_TI814X_MUXENTRY(MCASP5_FSX, 0,
			"mcasp5_fsx", NULL, NULL, "tso0_bytstrt",
			NULL, NULL, NULL, "gpio0_26_mux1"),
	_TI814X_MUXENTRY(MCASP5_AXR0, 0,
			"mcasp5_axr0", "mcasp4_axr2", NULL, "tso0_data",
			NULL, NULL, NULL, "gpio0_27_mux1"),
	_TI814X_MUXENTRY(MCASP5_AXR1, 0,
			"mcasp5_axr1", "mcasp4_axr3", NULL, "tso0_pacval",
			NULL, NULL, "timer7_mux0", "gpio0_28_mux1"),
	_TI814X_MUXENTRY(MLB_SIG, 0,
			"mlb_sig", NULL, NULL, "uart2_rxd_mux1",
			NULL, NULL, NULL, "gpio0_29"),
	_TI814X_MUXENTRY(MLB_DAT, 0,
			"mlb_dat", NULL, NULL, "tclkin",
			NULL, NULL, NULL, "gpio0_30"),
	_TI814X_MUXENTRY(MLB_CLK, 0,
			"mlb_clk", NULL, NULL, "uart2_txd_mux1",
			NULL, NULL, NULL, "gpio0_31"),
	_TI814X_MUXENTRY(MLBP_SIG_P, 0,
			"mlbp_sig_p", NULL, NULL, NULL,
			NULL, NULL, NULL, "gpio1_07_mux1"),
	_TI814X_MUXENTRY(MLBP_SIG_N, 0,
			NULL, NULL, NULL, NULL,
			NULL, NULL, NULL, "gpio1_08_mux1"),
	_TI814X_MUXENTRY(MLBP_DAT_P, 0,
			"mlbp_dat_p", NULL, NULL, NULL,
			NULL, NULL, NULL, "gpio1_09_mux1"),
	_TI814X_MUXENTRY(MLBP_DAT_N, 0,
			NULL, NULL, NULL, NULL,
			NULL, NULL, NULL, "gpio1_10_mux1"),
	_TI814X_MUXENTRY(MLBP_CLK_P, 0,
			"mlbp_clk_p", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(MLBP_CLK_N, 0,
			"mlbp_clk_n", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(DCAN0_TX, 0,
			"dcan0_tx", "uart2_txd_mux2", NULL, NULL, NULL,
			"i2c3_sda_mux1", NULL, "gpio1_0"),
	_TI814X_MUXENTRY(DCAN0_RX, 0,
			"dcan0_rx", "uart2_rxd_mux2", NULL, NULL, NULL,
			"i2c3_scl_mux1", NULL, "gpio1_1"),
	_TI814X_MUXENTRY(UART0_RXD, 0,
			"uart0_rxd", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(UART0_TXD, 0,
			"uart0_txd", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(UART0_CTSN, 0,
			"uart0_ctsn", "uart4_rxd_mux3", NULL, "dcan1_tx",
			"spi1_cs3", NULL, "mmc0_sdcd", NULL),
	_TI814X_MUXENTRY(UART0_RTSN, 0,
			"uart0_rtsn", "uart4_txd_mux3", NULL, "dcan1_rx",
			"spi1_cs2", NULL, "mmc2_sdcd", NULL),
	_TI814X_MUXENTRY(UART0_DCDN, 0,
			"uart0_dcdn", "uart3_rxd_mux0", NULL, NULL, "spi0_cs3",
			"i2c2_scl_mux0", "mmc1_pow", "gpio1_2"),
	_TI814X_MUXENTRY(UART0_DSRN, 0,
			"uart0_dsrn", "uart3_txd_mux0", NULL, NULL, "spi0_cs2",
			"i2c2_sda_mux0", "mmc1_sdwp", "gpio1_3"),
	_TI814X_MUXENTRY(UART0_DTRN, 0,
			"uart0_dtrn", "uart3_ctsn_mux0", "uart1_txd_mux0", NULL,
			NULL, NULL, NULL, "gpio1_4"),
	_TI814X_MUXENTRY(UART0_RIN, 0,
			"uart0_rin", "uart3_rtsn_mux0", "uart1_rxd_mux0", NULL,
			NULL, NULL, NULL, "gpio1_5"),
	_TI814X_MUXENTRY(I2C1_SCL, 0,
			"i2c1_scl", "hdmi_ddc_scl_mux0", NULL, NULL, NULL, NULL,
			NULL, NULL),
	_TI814X_MUXENTRY(I2C1_SDA, 0,
			"i2c1_sda", "hdmi_ddc_sda_mux0", NULL, NULL, NULL, NULL,
			NULL, NULL),
	_TI814X_MUXENTRY(SPI0_CS1, 0,
			"spi0_cs1", "mmc1_sdcd", "sata_act0_led", NULL, NULL,
			"xdma_evt_1_mux1", "timer4_mux1", "gpio1_6"),
	_TI814X_MUXENTRY(SPI0_CS0, 0,
			"spi0_cs0", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(SPI0_SCLK, 0,
			"spi0_sclk", NULL, NULL, NULL, NULL, NULL, "`", NULL),
	_TI814X_MUXENTRY(SPI0_D1, 0,
			"spi0_d1", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(SPI0_D0, 0,
			"spi0_d0", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(SPI1_CS0, 0,
			"spi1_cs0", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_16_mux1"),
	_TI814X_MUXENTRY(SPI1_SCLK, 0,
			"spi1_sclk", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_17_mux1"),
	_TI814X_MUXENTRY(SPI1_D1, 0,
			"spi1_d1", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_18_mux1"),
	_TI814X_MUXENTRY(SPI1_D0, 0,
			"spi1_d0", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_26_mux1"),
	_TI814X_MUXENTRY(GPMC_AD00, 0,
			"gpmc_ad0", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot0"),
	_TI814X_MUXENTRY(GPMC_AD01, 0,
			"gpmc_ad1", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot1"),
	_TI814X_MUXENTRY(GPMC_AD02, 0,
			"gpmc_ad2", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot2"),
	_TI814X_MUXENTRY(GPMC_AD03, 0,
			"gpmc_ad3", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot3"),
	_TI814X_MUXENTRY(GPMC_AD04, 0,
			"gpmc_ad4", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot4"),
	_TI814X_MUXENTRY(GPMC_AD05, 0,
			"gpmc_ad5", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot5"),
	_TI814X_MUXENTRY(GPMC_AD06, 0,
			"gpmc_ad6", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot6"),
	_TI814X_MUXENTRY(GPMC_AD07, 0,
			"gpmc_ad7", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot7"),
	_TI814X_MUXENTRY(GPMC_AD08, 0,
			"gpmc_ad8", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot8"),
	_TI814X_MUXENTRY(GPMC_AD09, 0,
			"gpmc_ad9", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot9"),
	_TI814X_MUXENTRY(GPMC_AD10, 0,
			"gpmc_ad10", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot10"),
	_TI814X_MUXENTRY(GPMC_AD11, 0,
			"gpmc_ad11", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot11"),
	_TI814X_MUXENTRY(GPMC_AD12, 0,
			"gpmc_ad12", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot12"),
	_TI814X_MUXENTRY(GPMC_AD13, 0,
			"gpmc_ad13", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot13"),
	_TI814X_MUXENTRY(GPMC_AD14, 0,
			"gpmc_ad14", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot14"),
	_TI814X_MUXENTRY(GPMC_AD15, 0,
			"gpmc_ad15", NULL, NULL, NULL, NULL, NULL, NULL,
			"boot15"),
	_TI814X_MUXENTRY(GPMC_A16, 0,
			"gpmc_a16", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_05_mux0"),
	_TI814X_MUXENTRY(GPMC_A17, 0,
			"gpmc_a17", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_06_mux0"),
	_TI814X_MUXENTRY(GPMC_A18, 0,
			"gpmc_a18", NULL, NULL, NULL, NULL, NULL, "timer2_mux2",
			"gpio1_13_mux0"),
	_TI814X_MUXENTRY(GPMC_A19, 0,
			"gpmc_a19", NULL, NULL, NULL, NULL, NULL, "timer3_mux2",
			"gpio1_14_mux0"),
	_TI814X_MUXENTRY(GPMC_A20, 0,
			"gpmc_a_20_mux0", NULL, "spi2_cs1", NULL, NULL, NULL,
			NULL, "gpio1_15_mux0"),
	_TI814X_MUXENTRY(GPMC_A21, 0,
			"gpmc_a_21_mux0", NULL, "spi2_d_0_mux0", NULL, NULL,
			NULL, NULL, "gpio1_16_mux0"),
	_TI814X_MUXENTRY(GPMC_A22, 0,
			"gpmc_a_22_mux0", NULL, "spi2_d_1_mux0", NULL,
			"hdmi_cec_mux0", NULL, "timer4_mux2", "gpio1_17_mux0"),
	_TI814X_MUXENTRY(GPMC_A23, 0,
			"gpmc_a_23_mux0", NULL, "spi2_sclk_mux0", NULL,
			"hdmi_hpd_mux0", NULL, "timer5_mux2", "gpio1_18_mux0"),
	_TI814X_MUXENTRY(MMC2_DAT7, 0,
			"mmc2_dat7", "gpmc_a_24_mux0", "gpmc_a_20_mux1", NULL,
			NULL, "uart2_rxd_mux3", NULL, "gpio1_19"),
	_TI814X_MUXENTRY(MMC2_DAT6, 0,
			"mmc2_dat6", "gpmc_a_25_mux0", "gpmc_a_21_mux1", NULL,
			NULL, "uart2_txd_mux3", NULL, "gpio1_20"),
	_TI814X_MUXENTRY(MMC2_DAT5, 0,
			"mmc2_dat5", "gpmc_a_26_mux0", "gpmc_a_22_mux1", NULL,
			NULL, NULL, "timer6_mux2", "gpio1_21"),
	_TI814X_MUXENTRY(MMC2_DAT4, 0,
			"mmc2_dat4", "gpmc_a_27_mux0", "gpmc_a_23_mux1",
			"gpmc_cs7", NULL, "xdma_evt_0_mux1", "timer7_mux2",
			"gpio1_22"),
	_TI814X_MUXENTRY(MMC2_DAT3, 0,
			"mmc2_dat3", "gpmc_a_1_mux1", NULL, NULL, NULL, NULL,
			NULL, "gpio2_05_mux1"),
	_TI814X_MUXENTRY(MMC2_DAT2, 0,
			"mmc2_dat2", "gpmc_a_2_mux1", NULL, NULL, NULL, NULL,
			NULL, "gpio2_06_mux1"),
	_TI814X_MUXENTRY(MMC2_DAT1, 0,
			"mmc2_dat1", "gpmc_a_3_mux1", NULL, NULL, NULL, NULL,
			NULL, "gpio1_13_mux1"),
	_TI814X_MUXENTRY(MMC2_DAT0, 0,
			"mmc2_dat0", "gpmc_a_4_mux1", NULL, NULL, NULL, NULL,
			NULL, "gpio1_14_mux1"),
	_TI814X_MUXENTRY(MMC2_CLK, 0,
			"mmc2_clk", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_15_mux1"),
	_TI814X_MUXENTRY(GPMC_CS0, 0,
			"gpmc_cs0", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_23"),
	_TI814X_MUXENTRY(GPMC_CS1, 0,
			"gpmc_cs1", "gpmc_a_25_mux1", NULL, NULL, NULL, NULL,
			NULL, "gpio1_24"),
	_TI814X_MUXENTRY(GPMC_CS2, 0,
			"gpmc_cs2", "gpmc_a_24_mux1", NULL, NULL, NULL, NULL,
			NULL, "gpio1_25"),
	_TI814X_MUXENTRY(GPMC_CS3, 0,
			"gpmc_cs3", "vin1_clk1", "spi2_cs0", NULL, NULL, NULL,
			NULL, "gpio1_26_mux0"),
	_TI814X_MUXENTRY(GPMC_CS4, 0,
			"gpmc_cs4", "mmc2_cmd", NULL, NULL, NULL, NULL, NULL,
			"gpio1_08_mux0"),
	_TI814X_MUXENTRY(GPMC_CLK, 0,
			"gpmc_clk", "gpmc_cs5", NULL, "gpmc_wait1", "clkout1",
			"xdma_evt_3_mux0", "timer4_mux3", "gpio1_27"),
	_TI814X_MUXENTRY(GPMC_ADVN_ALE, 0,
			"gpmc_advn_ale", "gpmc_cs6", NULL, NULL, NULL, NULL,
			"timer5_mux3", "gpio1_28"),
	_TI814X_MUXENTRY(GPMC_OEN_REN, 0,
			"gpmc_oen_ren", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(GPMC_WEN, 0,
			"gpmc_wen", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(GPMC_BEN0, 0,
			"gpmc_ben0", "gpmc_a_25_mux2", NULL, NULL, NULL,
			"xdma_evt_2_mux0", "timer6_mux3", "gpio1_29"),
	_TI814X_MUXENTRY(GPMC_BEN1, 0,
			"gpmc_ben1", "gpmc_a_24_mux2", NULL, NULL, NULL,
			"xdma_evt_1_mux0", "timer7_mux3", "gpio1_30"),
	_TI814X_MUXENTRY(GPMC_WAIT0, 0,
			"gpmc_wait0", "gpmc_a_26_mux2", NULL, NULL, NULL,
			"xdma_evt_0_mux0", NULL, "gpio1_31"),
	_TI814X_MUXENTRY(VIN0_CLK1, 0,
			"vin0_clk1", NULL, NULL, NULL, NULL, "cam_xclka", NULL,
			"gpio1_09_mux0"),
	_TI814X_MUXENTRY(VIN0_DE0_MUX0, 0,
			"vin0_de0_mux0", NULL, NULL, NULL, "vin0_hsync1",
			"uart5_txd_mux1", "i2c2_sda_mux1", "gpio2_0"),
	_TI814X_MUXENTRY(VIN0_FLD0_MUX0, 0,
			"vin0_fld0_mux0", NULL, NULL, NULL, "vin0_vsync1",
			"uart5_rxd_mux1", "i2c2_scl_mux3", "gpio2_1"),
	_TI814X_MUXENTRY(VIN0_CLK0, 0,
			"vin0_clk0", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_02_mux1"),
	_TI814X_MUXENTRY(VIN0_HSYNC0, 0,
			"vin0_hsync0", NULL, NULL, NULL, NULL,
			"uart5_rtsn_mux1", NULL, "gpio2_3"),
	_TI814X_MUXENTRY(VIN0_VSYNC0, 0,
			"vin0_vsync0", NULL, NULL, NULL, NULL,
			"uart5_ctsn_mux1", NULL, "gpio2_4"),
	_TI814X_MUXENTRY(VIN0_D0, 0,
			"vin0_d0", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_11_mux1"),
	_TI814X_MUXENTRY(VIN0_D1, 0,
			"vin0_d1", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_12_mux1"),
	_TI814X_MUXENTRY(VIN0_D2, 0,
			"vin0_d2", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_7"),
	_TI814X_MUXENTRY(VIN0_D3, 0,
			"vin0_d3", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_8"),
	_TI814X_MUXENTRY(VIN0_D4, 0,
			"vin0_d4", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_9"),
	_TI814X_MUXENTRY(VIN0_D5, 0,
			"vin0_d5", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_10"),
	_TI814X_MUXENTRY(VIN0_D6, 0,
			"vin0_d6", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_11"),
	_TI814X_MUXENTRY(VIN0_D7, 0,
			"vin0_d7", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_12"),
	_TI814X_MUXENTRY(VIN0_D8, 0,
			"vin0_d8", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_13"),
	_TI814X_MUXENTRY(VIN0_D9, 0,
			"vin0_d9", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_14"),
	_TI814X_MUXENTRY(VIN0_D10, 0,
			"vin0_d10", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_15"),
	_TI814X_MUXENTRY(VIN0_D11, 0,
			"vin0_d11", NULL, NULL, NULL, NULL, "cam_de_mux1", NULL,
			"gpio2_16"),
	_TI814X_MUXENTRY(VIN0_D12, 0,
			"vin0_d12", NULL, NULL, NULL, NULL, "cam_xclkb", NULL,
			"gpio2_17"),
	_TI814X_MUXENTRY(VIN0_D13, 0,
			"vin0_d13", NULL, NULL, NULL, NULL, "cam_reset", NULL,
			"gpio2_18"),
	_TI814X_MUXENTRY(VIN0_D14, 0,
			"vin0_d14", NULL, NULL, NULL, NULL, "cam_strobe", NULL,
			"gpio2_19"),
	_TI814X_MUXENTRY(VIN0_D15, 0,
			"vin0_d15", NULL, NULL, NULL, NULL, "cam_shutter", NULL,
			"gpio2_20"),
	_TI814X_MUXENTRY(VIN0_D16, 0,
			"vin0_d16", "cam_d8", NULL, NULL, NULL, "i2c2_scl_mux1",
			NULL, "gpio0_10_mux0"),
	_TI814X_MUXENTRY(VIN0_D17, 0,
			"vin0_d17", "cam_d9", NULL, "rmii1_rxer_mux1", NULL,
			NULL, NULL, "gpio0_11_mux0"),
	_TI814X_MUXENTRY(VIN0_D18, 0,
			"vin0_d18", "cam_d10", NULL, "rmii1_rxd1_mux1", NULL,
			"i2c3_scl_mux2", NULL, "gpio0_12_mux0"),
	_TI814X_MUXENTRY(VIN0_D19, 0,
			"vin0_d19", "cam_d11", NULL, "rmii1_rxd0_mux1", NULL,
			"i2c3_sda_mux2", NULL, "gpio0_13_mux0"),
	_TI814X_MUXENTRY(VIN0_D20, 0,
			"vin0_d20", "cam_d12", NULL, "rmii1_crs_mux1", NULL,
			"spi3_cs0", NULL, "gpio0_14_mux0"),
	_TI814X_MUXENTRY(VIN0_D21, 0,
			"vin0_d21", "cam_d13", NULL, "rmii1_txd0_mux1", NULL,
			"spi3_sclk_mux0", NULL, "gpio0_15_mux0"),
	_TI814X_MUXENTRY(VIN0_D22, 0,
			"vin0_d22", "cam_d14", NULL, "rmii1_txd1_mux1", NULL,
			"spi3_d_1_mux0", NULL, "gpio0_16_mux0"),
	_TI814X_MUXENTRY(VIN0_D23, 0,
			"vin0_d23", "cam_d15", NULL, "rmii1_txen_mux1", NULL,
			"spi3_d_0_mux0", NULL, "gpio0_17_mux0"),
	_TI814X_MUXENTRY(VIN0_DE0_MUX1, 0,
			"vin0_de0_mux1", "cam_d7", NULL, "pata_diorn", NULL,
			NULL, NULL, "gpio0_18_mux0"),
	_TI814X_MUXENTRY(VIN0_DE1, 0,
			"vin0_de1", "cam_d6", NULL, "pata_csn1", NULL, NULL,
			NULL, "gpio0_19_mux0"),
	_TI814X_MUXENTRY(VIN0_FLD0_MUX1, 0,
			"vin0_fld0_mux1", "cam_d5", NULL, "pata_csn0", NULL,
			NULL, NULL, "gpio0_20_mux0"),
	_TI814X_MUXENTRY(VIN0_FLD1, 0,
			"vin0_fld1", "cam_d4", NULL, "pata_diown", NULL, NULL,
			NULL, "gpio0_21_mux0"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC1, 0,
			"vout1_g_y_yc1", "cam_d3", NULL, "pata_dmackn",
			"gpmc_a_5_mux1", "uart4_rxd_mux0", NULL,
			"gpio0_22_mux0"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC0, 0,
			"vout1_g_y_yc0", "cam_d2", NULL, "pata_da2",
			"gpmc_a_6_mux1", "uart4_txd_mux0", NULL,
			"gpio0_23_mux0"),
	_TI814X_MUXENTRY(VOUT1_R_CR1, 0,
			"vout1_r_cr1", "cam_d1", NULL, "pata_da1",
			"gpmc_a_7_mux1", "uart4_ctsn_mux0", NULL,
			"gpio0_24_mux0"),
	_TI814X_MUXENTRY(VOUT1_R_CR0, 0,
			"vout1_r_cr0", "cam_d0", NULL, "pata_da0",
			"gpmc_a_8_mux1", "uart4_rtsn_mux0", NULL,
			"gpio0_25_mux0"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C1, 0,
			"vout1_b_cb_c1", "cam_hs", NULL, "pata_dmarq",
			"gpmc_a_9_mux1", "uart2_rxd_mux0", NULL,
			"gpio0_26_mux0"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C0, 0,
			"vout1_b_cb_c0", "cam_vs", NULL, "pata_iordy",
			"gpmc_a_10_mux1", "uart2_txd_mux0", NULL,
			"gpio0_27_mux0"),
	_TI814X_MUXENTRY(VOUT1_FID, 0,
			"vout1_fid", "cam_fld", "cam_de_mux0", "pata_intrq",
			"gpmc_a_11_mux1", "uart2_ctsn", NULL, "gpio0_28_mux0"),
	_TI814X_MUXENTRY(VOUT0_FID, 0,
			"vout0_fid_mux1", "cam_pclk", NULL, "pata_resetn",
			"gpmc_a_12_mux1", "uart2_rtsn", NULL, "gpio2_02_mux0"),
	_TI814X_MUXENTRY(VOUT0_CLK, 0,
			"vout0_clk", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(VOUT0_HSYNC, 0,
			"vout0_hsync", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_VSYNC, 0,
			"vout0_vsync", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_AVID, 0,
			"vout0_avid", "vout0_fid_mux0", NULL, NULL,
			"spi3_sclk_mux2", NULL, "timer7_mux1", "gpio2_21"),
	_TI814X_MUXENTRY(VOUT0_B_CB_C2, 0,
			"vout0_b_cb_c2", "emu2", NULL, NULL, NULL, NULL, NULL,
			"gpio2_22"),
	_TI814X_MUXENTRY(VOUT0_B_CB_C3, 0,
			"vout0_b_cb_c3", "obs_event0", NULL, NULL, NULL, NULL,
			NULL, "gpio2_23"),
	_TI814X_MUXENTRY(VOUT0_B_CB_C4, 0,
			"vout0_b_cb_c4", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_B_CB_C5, 0,
			"vout0_b_cb_c5", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_B_CB_C6, 0,
			"vout0_b_cb_c6", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_B_CB_C7, 0,
			"vout0_b_cb_c7", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_B_CB_C8, 0,
			"vout0_b_cb_c8", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_B_CB_C9, 0,
			"vout0_b_cb_c9", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC2, 0,
			"vout0_g_y_yc2", "emu3", NULL, NULL, NULL, NULL, NULL,
			"gpio2_24"),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC3, 0,
			"vout0_g_y_yc3", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio2_25"),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC4, 0,
			"vout0_g_y_yc4", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC5, 0,
			"vout0_g_y_yc5", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC6, 0,
			"vout0_g_y_yc6", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC7, 0,
			"vout0_g_y_yc7", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC8, 0,
			"vout0_g_y_yc8", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_G_Y_YC9, 0,
			"vout0_g_y_yc9", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_R_CR2, 0,
			"vout0_r_cr2", "emu4", NULL, NULL, NULL, NULL, NULL,
			"gpio2_26"),
	_TI814X_MUXENTRY(VOUT0_R_CR3, 0,
			"vout0_r_cr3", "obs_event1", NULL, NULL, NULL, NULL,
			NULL, "gpio2_27"),
	_TI814X_MUXENTRY(VOUT0_R_CR4, 0,
			"vout0_r_cr4", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_R_CR5, 0,
			"vout0_r_cr5", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_R_CR6, 0,
			"vout0_r_cr6", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_R_CR7, 0,
			"vout0_r_cr7", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_R_CR8, 0,
			"vout0_r_cr8", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT0_R_CR9, 0,
			"vout0_r_cr9", NULL, NULL, NULL, NULL, NULL, NULL,
			NULL),
	_TI814X_MUXENTRY(VOUT1_CLK, 0,
			"vout1_clk", "gmii1_txclk", "vin1_hsync0",
			"pata_hddir_mux0", NULL, NULL, NULL, "gpio2_28"),
	_TI814X_MUXENTRY(VOUT1_HSYNC, 0,
			"vout1_hsync", "gmii1_col", "vin1_vsync0",
			"pata_hddir_mux1", "spi3_d_1_mux2", "uart3_rtsn_mux1",
			NULL, "gpio2_29"),
	_TI814X_MUXENTRY(VOUT1_VSYNC, 0,
			"vout1_vsync", "gmii1_crs", "vin1_fid0", "vin1_de0",
			"spi3_d_0_mux2", "uart3_ctsn_mux1", NULL, "gpio2_30"),
	_TI814X_MUXENTRY(VOUT1_AVID, 0,
			"vout1_avid", "gmii1_rxer", "vin1_clk0", NULL, NULL,
			"uart4_rtsn_mux2", "timer6_mux1", "gpio2_31"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C3, 0,
			"vout1_b_cb_c3", "gmii1_rxclk", "vin1a_d0", NULL, NULL,
			"uart4_ctsn_mux2", NULL, "gpio3_0"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C4, 0,
			"vout1_b_cb_c4", "gmii1_rxd0", "vin1a_d1", NULL, NULL,
			"uart4_rxd_mux2", NULL, "gpio3_1"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C5, 0,
			"vout1_b_cb_c5", "gmii1_rxd1", "vin1a_d2", NULL, NULL,
			"uart4_txd_mux2", NULL, "gpio3_2"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C6, 0,
			"vout1_b_cb_c6", "gmii1_rxd2", "vin1a_d3", NULL, NULL,
			"uart3_rxd_mux1", NULL, "gpio3_3"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C7, 0,
			"vout1_b_cb_c7", "gmii1_rxd3", "vin1a_d4", NULL, NULL,
			"uart3_txd_mux1", NULL, "gpio3_4"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C8, 0,
			"vout1_b_cb_c8", "gmii1_rxd4", "vin1a_d5", NULL, NULL,
			"i2c3_scl_mux3", NULL, "gpio3_5"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C9, 0,
			"vout1_b_cb_c9", "gmii1_rxd5", "vin1a_d6", NULL, NULL,
			"i2c3_sda_mux3", NULL, "gpio3_6"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC3, 0,
			"vout1_g_y_yc3", "gmii1_rxd6", "vin1a_d8", "pata_d0",
			NULL, NULL, NULL, "gpio3_7"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC4, 0,
			"vout1_g_y_yc4", "gmii1_rxd7", "vin1a_d9", "pata_d1",
			NULL, NULL, NULL, "gpio3_8"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC5, 0,
			"vout1_g_y_yc5", "gmii1_rxdv", "vin1a_d10", "pata_d2",
			NULL, NULL, NULL, "gpio3_9"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC6, 0,
			"vout1_g_y_yc6", "gmii1_gtxclk", "vin1a_d11", "pata_d3",
			NULL, NULL, NULL, "gpio3_10"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC7, 0,
			"vout1_g_y_yc7", "gmii1_txd0", "vin1a_d12", "pata_d4",
			NULL, NULL, NULL, "gpio3_11"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC8, 0,
			"vout1_g_y_yc8", "gmii1_txd1", "vin1a_d13", "pata_d5",
			NULL, NULL, NULL, "gpio3_12"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC9, 0,
			"vout1_g_y_yc9", "gmii1_txd2", "vin1a_d14", "pata_d6",
			NULL, NULL, NULL, "gpio3_13"),
	_TI814X_MUXENTRY(VOUT1_R_CR4, 0,
			"vout1_r_cr4", "gmii1_txd3", "vin1a_d15", "pata_d7",
			NULL, "spi3_cs1", NULL, "gpio3_14"),
	_TI814X_MUXENTRY(VOUT1_R_CR5, 0,
			"vout1_r_cr5", "gmii1_txd4", "vin1a_d16", "pata_d8",
			NULL, "spi3_sclk_mux1", NULL, "gpio3_15"),
	_TI814X_MUXENTRY(VOUT1_R_CR6, 0,
			"vout1_r_cr6", "gmii1_txd5", "vin1a_d17", "pata_d9",
			NULL, "spi3_d_1_mux1", NULL, "gpio3_16"),
	_TI814X_MUXENTRY(VOUT1_R_CR7, 0,
			"vout1_r_cr7", "gmii1_txd6", "vin1a_d18", "pata_d10",
			NULL, "spi3_d_0_mux1", NULL, "gpio3_17"),
	_TI814X_MUXENTRY(VOUT1_R_CR8, 0,
			"vout1_r_cr8", "gmii1_txd7", "vin1a_d19", "pata_d11",
			NULL, "uart5_rxd_mux2", NULL, "gpio3_18"),
	_TI814X_MUXENTRY(VOUT1_R_CR9, 0,
			"vout1_r_cr9", "gmii1_txen", "vin1a_d20", "pata_d12",
			NULL, "uart5_txd_mux2", NULL, "gpio3_19"),
	_TI814X_MUXENTRY(VOUT1_G_Y_YC2, 0,
			"vout1_g_y_yc2", "gpmc_a_13_mux1", "vin1a_d21",
			"pata_d13", "hdmi_ddc_scl_mux1", "spi2_cs2",
			"i2c2_scl_mux2", "gpio3_20"),
	_TI814X_MUXENTRY(VOUT1_R_CR3, 0,
			"vout1_r_cr3", "gpmc_a_14_mux1", "vin1a_d22",
			"pata_d14", "hdmi_ddc_sda_mux1", "spi2_sclk_mux1",
			"i2c2_sda_mux2", "gpio3_21"),
	_TI814X_MUXENTRY(VOUT1_R_CR2, 0,
			"vout1_r_cr2", "gpmc_a_15_mux1", "vin1a_d23",
			"pata_d15", "hdmi_hpd_mux1", "spi2_d_1_mux1", NULL,
			"gpio3_22"),
	_TI814X_MUXENTRY(VOUT1_B_CB_C2, 0,
			"vout1_b_cb_c2", "gpmc_a_0_mux1", "vin1a_d7", NULL,
			"hdmi_cec_mux1", "spi2_d_0_mux1", NULL,
			"gpio3_30_mux1"),
	_TI814X_MUXENTRY(RMII_REFCLK, 0,
			"rmii_refclk", NULL, NULL, NULL, NULL, NULL,
			"timer2_mux3", "gpio1_10_mux0"),
	_TI814X_MUXENTRY(MDIO_MCLK, 0,
			"mdio_mclk", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_11_mux0"),
	_TI814X_MUXENTRY(MDIO_D, 0,
			"mdio_d", NULL, NULL, NULL, NULL, NULL, NULL,
			"gpio1_12_mux0"),
	_TI814X_MUXENTRY(GMII0_TXCLK, 0,
			"gmii0_txclk", "vin1b_d0", NULL, NULL, NULL, "spi3_cs3",
			"i2c2_sda_mux3", "gpio3_23"),
	_TI814X_MUXENTRY(GMII0_COL, 0,
			"gmii0_col", "vin1b_d1", "rmii0_rxd0", NULL, NULL, NULL,
			NULL, "gpio3_24"),
	_TI814X_MUXENTRY(GMII0_CRS, 0,
			"gmii0_crs", "vin1b_d2", "rmii0_rxd1", NULL, NULL, NULL,
			NULL, "gpio3_25"),
	_TI814X_MUXENTRY(GMII0_RXER, 0,
			"gmii0_rxer", "vin1b_d3", "rmii0_rxer", NULL, NULL,
			NULL, NULL, "gpio3_26"),
	_TI814X_MUXENTRY(GMII0_RXCLK, 0,
			"gmii0_rxclk", "vin1b_d4", "rmii0_crs", NULL, NULL,
			"spi3_cs2", NULL, "gpio3_27"),
	_TI814X_MUXENTRY(GMII0_RXD0, 0,
			"gmii0_rxd0", "vin1b_d5", "rmii0_txd0", NULL, NULL,
			NULL, NULL, "gpio3_28"),
	_TI814X_MUXENTRY(GMII0_RXD1, 0,
			"gmii0_rxd1", "vin1b_d6", "rmii0_txd1", NULL, NULL,
			NULL, NULL, "gpio3_29"),
	_TI814X_MUXENTRY(GMII0_RXD2, 0,
			"gmii0_rxd2", "vin1b_d7", "rmii0_txen", NULL, NULL,
			NULL, NULL, "gpio3_30_mux0"),
	_TI814X_MUXENTRY(GMII0_RXD3, 0,
			"gmii0_rxd3", NULL, "gpmc_a_27_mux1", "gpmc_a_26_mux1",
			"gpmc_a_0_mux0", "uart5_rxd_mux0", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_RXD4, 0,
			"gmii0_rxd4", NULL, NULL, NULL, "gpmc_a_1_mux0",
			"uart5_txd_mux0", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_RXD5, 0,
			"gmii0_rxd5", NULL, NULL, NULL, "gpmc_a_2_mux0",
			"uart5_ctsn_mux0", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_RXD6, 0,
			"gmii0_rxd6", NULL, NULL, NULL, "gpmc_a_3_mux0",
			"uart5_rtsn_mux0", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_RXD7, 0,
			"gmii0_rxd7", NULL, NULL, NULL, "gpmc_a_4_mux0",
			"spi2_cs3", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_RXDV, 0,
			"gmii0_rxdv", NULL, NULL, NULL, "gpmc_a_5_mux0",
			"spi2_sclk_mux2", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_GTXCLK, 0,
			"gmii0_gtxclk", NULL, NULL, NULL, "gpmc_a_6_mux0",
			"spi2_d_1_mux2", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD0, 0,
			"gmii0_txd0", NULL, NULL, NULL, "gpmc_a_7_mux0",
			"spi2_d_0_mux2", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD1, 0,
			"gmii0_txd1", NULL, NULL, NULL, "gpmc_a_8_mux0",
			"uart4_rxd_mux1", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD2, 0,
			"gmii0_txd2", "rmii1_rxd0_mux0", NULL, NULL,
			"gpmc_a_9_mux0", "uart4_txd_mux1", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD3, 0,
			"gmii0_txd3", "rmii1_rxd1_mux0", NULL, NULL,
			"gpmc_a_10_mux0", "uart4_ctsn_mux1", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD4, 0,
			"gmii0_txd4", "rmii1_rxer_mux0", NULL, NULL,
			"gpmc_a_11_mux0", "uart4_rtsn_mux1", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD5, 0,
			"gmii0_txd5", "rmii1_crs_mux0", NULL, NULL,
			"gpmc_a_12_mux0", "uart1_rxd_mux1", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD6, 0,
			"gmii0_txd6", "rmii1_txd0_mux0", NULL, NULL,
			"gpmc_a_13_mux0", "uart1_txd_mux1", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXD7, 0,
			"gmii0_txd7", "rmii1_txd1_mux0", NULL, NULL,
			"gpmc_a_14_mux0", "uart1_ctsn", NULL, NULL),
	_TI814X_MUXENTRY(GMII0_TXEN, 0,
			"gmii0_txen", "rmii1_txen_mux0", NULL, NULL,
			"gpmc_a_15_mux0", "uart1_rtsn", NULL, NULL),
	_TI814X_MUXENTRY(CLKIN32, 0,
			"clkin32", NULL, "clkout0", NULL, NULL, NULL,
			"timer3_mux3", "gpio3_31"),
	_TI814X_MUXENTRY(RESETN, 0,
			"resetn", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(NMIN, 0,
			"nmin", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(RSTOUTN, 0,
			"rstoutn", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(I2C0_SCL, 0,
			"i2c0_scl", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(I2C0_SDA, 0,
			"i2c0_sda", NULL, NULL, NULL, NULL, NULL, NULL, NULL),
	_TI814X_MUXENTRY(USB_DRVVBUS, 0,
			"usb0_drvvbus", "gpio0_7", NULL, NULL, NULL, NULL, NULL,
			NULL),
	{ .reg_offset = OMAP_MUX_TERMINATOR },
};

int __init ti814x_mux_init(struct omap_board_mux *board_subset)
{
	return omap_mux_init("core", OMAP_MUX_REG_32BIT,
			TI81XX_CONTROL_PADCONF_MUX_PBASE,
			TI814X_CONTROL_PADCONF_MUX_SIZE, ti814x_muxmodes, NULL,
			board_subset, NULL);
}
#else
int __init ti814x_mux_init(struct omap_board_mux *board_subset)
{
	return 0;
}
#endif
