/*
 * TI814x pad control register macros.
 *
 * Copyright (C) 2010 Texas Instruments, Inc. - http://www.ti.com/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __ARCH_ARM_MACH_OMAP2_MUX814X_H
#define __ARCH_ARM_MACH_OMAP2_MUX814X_H

#define TI81XX_CONTROL_PADCONF_MUX_PBASE			0x48140000LU

#define TI814X_MUX(mode0, mux_val)					\
{									\
	.reg_offset	= (TI814X_CONTROL_PADCONF_##mode0##_OFFSET),	\
	.value		= (mux_val & ~0xff) | (1 << (mux_val & 0x7)),	\
}

/*
 * TI814X CONTROL_PADCONF* register offsets for pin-muxing
 *
 * Extracted from the spec. Add TI814X_CONTROL_PADCONF_MUX_PBASE to these values
 * to get the absolute addresses. The macro names below are mode-0 names of
 * corresponding pins.
 */
#define TI814X_CONTROL_PADCONF_MMC1_CLK_OFFSET			0x0800
#define TI814X_CONTROL_PADCONF_MMC1_CMD_OFFSET			0x0804
#define TI814X_CONTROL_PADCONF_MMC1_DAT0_OFFSET			0x0808
#define TI814X_CONTROL_PADCONF_MMC1_DAT1_OFFSET			0x080C
#define TI814X_CONTROL_PADCONF_MMC1_DAT2_OFFSET			0x0810
#define TI814X_CONTROL_PADCONF_MMC1_DAT3_OFFSET			0x0814
#define TI814X_CONTROL_PADCONF_OSC_WAKE_OFFSET			0x0818
#define TI814X_CONTROL_PADCONF_MMC0_CLK_OFFSET			0x081C
#define TI814X_CONTROL_PADCONF_MMC0_CMD_OFFSET			0x0820
#define TI814X_CONTROL_PADCONF_MMC0_DAT0_OFFSET			0x0824
#define TI814X_CONTROL_PADCONF_MMC0_DAT1_OFFSET			0x0828
#define TI814X_CONTROL_PADCONF_MMC0_DAT2_OFFSET			0x082C
#define TI814X_CONTROL_PADCONF_MMC0_DAT3_OFFSET			0x0830
#define TI814X_CONTROL_PADCONF_XREF_CLK0_OFFSET			0x0834
#define TI814X_CONTROL_PADCONF_XREF_CLK1_OFFSET			0x0838
#define TI814X_CONTROL_PADCONF_XREF_CLK2_OFFSET			0x083C
#define TI814X_CONTROL_PADCONF_MCASP0_ACLKX_OFFSET		0x0840
#define TI814X_CONTROL_PADCONF_MCASP0_FSX_OFFSET		0x0844
#define TI814X_CONTROL_PADCONF_MCASP0_CLKR_OFFSET		0x0848
#define TI814X_CONTROL_PADCONF_MCASP0_FSR_OFFSET		0x084C
#define TI814X_CONTROL_PADCONF_MCASP0_AXR0_OFFSET		0x0850
#define TI814X_CONTROL_PADCONF_MCASP0_AXR1_OFFSET		0x0854
#define TI814X_CONTROL_PADCONF_MCASP0_AXR2_OFFSET		0x0858
#define TI814X_CONTROL_PADCONF_MCASP0_AXR3_OFFSET		0x085C
#define TI814X_CONTROL_PADCONF_MCASP0_AXR4_OFFSET		0x0860
#define TI814X_CONTROL_PADCONF_MCASP0_AXR5_OFFSET		0x0864
#define TI814X_CONTROL_PADCONF_MCASP0_AXR6_OFFSET		0x0868
#define TI814X_CONTROL_PADCONF_MCASP0_AXR7_OFFSET		0x086C
#define TI814X_CONTROL_PADCONF_MCASP0_AXR8_OFFSET		0x0870
#define TI814X_CONTROL_PADCONF_MCASP0_AXR9_OFFSET		0x0874
#define TI814X_CONTROL_PADCONF_MCASP1_ACLKX_OFFSET		0x0878
#define TI814X_CONTROL_PADCONF_MCASP1_FSX_OFFSET		0x087C
#define TI814X_CONTROL_PADCONF_MCASP1_CLKR_OFFSET		0x0880
#define TI814X_CONTROL_PADCONF_MCASP1_FSR_OFFSET		0x0884
#define TI814X_CONTROL_PADCONF_MCASP1_AXR0_OFFSET		0x0888
#define TI814X_CONTROL_PADCONF_MCASP1_AXR1_OFFSET		0x088C
#define TI814X_CONTROL_PADCONF_MCASP1_AXR2_OFFSET		0x0890
#define TI814X_CONTROL_PADCONF_MCASP1_AXR3_OFFSET		0x0894
#define TI814X_CONTROL_PADCONF_MCASP2_CLKX_OFFSET		0x0898
#define TI814X_CONTROL_PADCONF_MCASP2_FSX_OFFSET		0x089C
#define TI814X_CONTROL_PADCONF_MCASP2_AXR0_OFFSET		0x08A0
#define TI814X_CONTROL_PADCONF_MCASP2_AXR1_OFFSET		0x08A4
#define TI814X_CONTROL_PADCONF_MCASP2_AXR2_OFFSET		0x08A8
#define TI814X_CONTROL_PADCONF_MCASP2_AXR3_OFFSET		0x08AC
#define TI814X_CONTROL_PADCONF_MCASP3_CLKX_OFFSET		0x08B0
#define TI814X_CONTROL_PADCONF_MCASP3_FSX_OFFSET		0x08B4
#define TI814X_CONTROL_PADCONF_MCASP3_AXR0_OFFSET		0x08B8
#define TI814X_CONTROL_PADCONF_MCASP3_AXR1_OFFSET		0x08BC
#define TI814X_CONTROL_PADCONF_MCASP3_AXR2_OFFSET		0x08C0
#define TI814X_CONTROL_PADCONF_MCASP3_AXR3_OFFSET		0x08C4
#define TI814X_CONTROL_PADCONF_MCASP4_CLKX_OFFSET		0x08C8
#define TI814X_CONTROL_PADCONF_MCASP4_FSX_OFFSET		0x08CC
#define TI814X_CONTROL_PADCONF_MCASP4_AXR0_OFFSET		0x08D0
#define TI814X_CONTROL_PADCONF_MCASP4_AXR1_OFFSET		0x08D4
#define TI814X_CONTROL_PADCONF_MCASP5_CLKX_OFFSET		0x08D8
#define TI814X_CONTROL_PADCONF_MCASP5_FSX_OFFSET		0x08DC
#define TI814X_CONTROL_PADCONF_MCASP5_AXR0_OFFSET		0x08E0
#define TI814X_CONTROL_PADCONF_MCASP5_AXR1_OFFSET		0x08E4
#define TI814X_CONTROL_PADCONF_MLB_SIG_OFFSET			0x08E8
#define TI814X_CONTROL_PADCONF_MLB_DAT_OFFSET			0x08EC
#define TI814X_CONTROL_PADCONF_MLB_CLK_OFFSET			0x08F0
#define TI814X_CONTROL_PADCONF_MLBP_SIG_P_OFFSET		0x08F4
#define TI814X_CONTROL_PADCONF_MLBP_SIG_N_OFFSET		0x08F8
#define TI814X_CONTROL_PADCONF_MLBP_DAT_P_OFFSET		0x08FC
#define TI814X_CONTROL_PADCONF_MLBP_DAT_N_OFFSET		0x0900
#define TI814X_CONTROL_PADCONF_MLBP_CLK_P_OFFSET		0x0904
#define TI814X_CONTROL_PADCONF_MLBP_CLK_N_OFFSET		0x0908
#define TI814X_CONTROL_PADCONF_DCAN0_TX_OFFSET			0x090C
#define TI814X_CONTROL_PADCONF_DCAN0_RX_OFFSET			0x0910
#define TI814X_CONTROL_PADCONF_UART0_RXD_OFFSET			0x0914
#define TI814X_CONTROL_PADCONF_UART0_TXD_OFFSET			0x0918
#define TI814X_CONTROL_PADCONF_UART0_CTSN_OFFSET		0x091C
#define TI814X_CONTROL_PADCONF_UART0_RTSN_OFFSET		0x0920
#define TI814X_CONTROL_PADCONF_UART0_DCDN_OFFSET		0x0924
#define TI814X_CONTROL_PADCONF_UART0_DSRN_OFFSET		0x0928
#define TI814X_CONTROL_PADCONF_UART0_DTRN_OFFSET		0x092C
#define TI814X_CONTROL_PADCONF_UART0_RIN_OFFSET			0x0930
#define TI814X_CONTROL_PADCONF_I2C1_SCL_OFFSET			0x0934
#define TI814X_CONTROL_PADCONF_I2C1_SDA_OFFSET			0x0938
#define TI814X_CONTROL_PADCONF_SPI0_CS1_OFFSET			0x093C
#define TI814X_CONTROL_PADCONF_SPI0_CS0_OFFSET			0x0940
#define TI814X_CONTROL_PADCONF_SPI0_SCLK_OFFSET			0x0944
#define TI814X_CONTROL_PADCONF_SPI0_D1_OFFSET			0x0948
#define TI814X_CONTROL_PADCONF_SPI0_D0_OFFSET			0x094C
#define TI814X_CONTROL_PADCONF_SPI1_CS0_OFFSET			0x0950
#define TI814X_CONTROL_PADCONF_SPI1_SCLK_OFFSET			0x0954
#define TI814X_CONTROL_PADCONF_SPI1_D1_OFFSET			0x0958
#define TI814X_CONTROL_PADCONF_SPI1_D0_OFFSET			0x095C
#define TI814X_CONTROL_PADCONF_GPMC_AD00_OFFSET			0x0960
#define TI814X_CONTROL_PADCONF_GPMC_AD01_OFFSET			0x0964
#define TI814X_CONTROL_PADCONF_GPMC_AD02_OFFSET			0x0968
#define TI814X_CONTROL_PADCONF_GPMC_AD03_OFFSET			0x096C
#define TI814X_CONTROL_PADCONF_GPMC_AD04_OFFSET			0x0970
#define TI814X_CONTROL_PADCONF_GPMC_AD05_OFFSET			0x0974
#define TI814X_CONTROL_PADCONF_GPMC_AD06_OFFSET			0x0978
#define TI814X_CONTROL_PADCONF_GPMC_AD07_OFFSET			0x097C
#define TI814X_CONTROL_PADCONF_GPMC_AD08_OFFSET			0x0980
#define TI814X_CONTROL_PADCONF_GPMC_AD09_OFFSET			0x0984
#define TI814X_CONTROL_PADCONF_GPMC_AD10_OFFSET			0x0988
#define TI814X_CONTROL_PADCONF_GPMC_AD11_OFFSET			0x098C
#define TI814X_CONTROL_PADCONF_GPMC_AD12_OFFSET			0x0990
#define TI814X_CONTROL_PADCONF_GPMC_AD13_OFFSET			0x0994
#define TI814X_CONTROL_PADCONF_GPMC_AD14_OFFSET			0x0998
#define TI814X_CONTROL_PADCONF_GPMC_AD15_OFFSET			0x099C
#define TI814X_CONTROL_PADCONF_GPMC_A16_OFFSET			0x09A0
#define TI814X_CONTROL_PADCONF_GPMC_A17_OFFSET			0x09A4
#define TI814X_CONTROL_PADCONF_GPMC_A18_OFFSET			0x09A8
#define TI814X_CONTROL_PADCONF_GPMC_A19_OFFSET			0x09AC
#define TI814X_CONTROL_PADCONF_GPMC_A20_OFFSET			0x09B0
#define TI814X_CONTROL_PADCONF_GPMC_A21_OFFSET			0x09B4
#define TI814X_CONTROL_PADCONF_GPMC_A22_OFFSET			0x09B8
#define TI814X_CONTROL_PADCONF_GPMC_A23_OFFSET			0x09BC
#define TI814X_CONTROL_PADCONF_MMC2_DAT7_OFFSET			0x09C0
#define TI814X_CONTROL_PADCONF_MMC2_DAT6_OFFSET			0x09C4
#define TI814X_CONTROL_PADCONF_MMC2_DAT5_OFFSET			0x09C8
#define TI814X_CONTROL_PADCONF_MMC2_DAT4_OFFSET			0x09CC
#define TI814X_CONTROL_PADCONF_MMC2_DAT3_OFFSET			0x09D0
#define TI814X_CONTROL_PADCONF_MMC2_DAT2_OFFSET			0x09D4
#define TI814X_CONTROL_PADCONF_MMC2_DAT1_OFFSET			0x09D8
#define TI814X_CONTROL_PADCONF_MMC2_DAT0_OFFSET			0x09DC
#define TI814X_CONTROL_PADCONF_MMC2_CLK_OFFSET			0x09E0
#define TI814X_CONTROL_PADCONF_GPMC_CS0_OFFSET			0x09E4
#define TI814X_CONTROL_PADCONF_GPMC_CS1_OFFSET			0x09E8
#define TI814X_CONTROL_PADCONF_GPMC_CS2_OFFSET			0x09EC
#define TI814X_CONTROL_PADCONF_GPMC_CS3_OFFSET			0x09F0
#define TI814X_CONTROL_PADCONF_GPMC_CS4_OFFSET			0x09F4
#define TI814X_CONTROL_PADCONF_GPMC_CLK_OFFSET			0x09F8
#define TI814X_CONTROL_PADCONF_GPMC_ADVN_ALE_OFFSET		0x09FC
#define TI814X_CONTROL_PADCONF_GPMC_OEN_REN_OFFSET		0x0A00
#define TI814X_CONTROL_PADCONF_GPMC_WEN_OFFSET			0x0A04
#define TI814X_CONTROL_PADCONF_GPMC_BEN0_OFFSET			0x0A08
#define TI814X_CONTROL_PADCONF_GPMC_BEN1_OFFSET			0x0A0C
#define TI814X_CONTROL_PADCONF_GPMC_WAIT0_OFFSET		0x0A10
#define TI814X_CONTROL_PADCONF_VIN0_CLK1_OFFSET			0x0A14
#define TI814X_CONTROL_PADCONF_VIN0_DE0_MUX0_OFFSET		0x0A18
#define TI814X_CONTROL_PADCONF_VIN0_FLD0_MUX0_OFFSET		0x0A1C
#define TI814X_CONTROL_PADCONF_VIN0_CLK0_OFFSET			0x0A20
#define TI814X_CONTROL_PADCONF_VIN0_HSYNC0_OFFSET		0x0A24
#define TI814X_CONTROL_PADCONF_VIN0_VSYNC0_OFFSET		0x0A28
#define TI814X_CONTROL_PADCONF_VIN0_D0_OFFSET			0x0A2C
#define TI814X_CONTROL_PADCONF_VIN0_D1_OFFSET			0x0A30
#define TI814X_CONTROL_PADCONF_VIN0_D2_OFFSET			0x0A34
#define TI814X_CONTROL_PADCONF_VIN0_D3_OFFSET			0x0A38
#define TI814X_CONTROL_PADCONF_VIN0_D4_OFFSET			0x0A3C
#define TI814X_CONTROL_PADCONF_VIN0_D5_OFFSET			0x0A40
#define TI814X_CONTROL_PADCONF_VIN0_D6_OFFSET			0x0A44
#define TI814X_CONTROL_PADCONF_VIN0_D7_OFFSET			0x0A48
#define TI814X_CONTROL_PADCONF_VIN0_D8_OFFSET			0x0A4C
#define TI814X_CONTROL_PADCONF_VIN0_D9_OFFSET			0x0A50
#define TI814X_CONTROL_PADCONF_VIN0_D10_OFFSET			0x0A54
#define TI814X_CONTROL_PADCONF_VIN0_D11_OFFSET			0x0A58
#define TI814X_CONTROL_PADCONF_VIN0_D12_OFFSET			0x0A5C
#define TI814X_CONTROL_PADCONF_VIN0_D13_OFFSET			0x0A60
#define TI814X_CONTROL_PADCONF_VIN0_D14_OFFSET			0x0A64
#define TI814X_CONTROL_PADCONF_VIN0_D15_OFFSET			0x0A68
#define TI814X_CONTROL_PADCONF_VIN0_D16_OFFSET			0x0A6C
#define TI814X_CONTROL_PADCONF_VIN0_D17_OFFSET			0x0A70
#define TI814X_CONTROL_PADCONF_VIN0_D18_OFFSET			0x0A74
#define TI814X_CONTROL_PADCONF_VIN0_D19_OFFSET			0x0A78
#define TI814X_CONTROL_PADCONF_VIN0_D20_OFFSET			0x0A7C
#define TI814X_CONTROL_PADCONF_VIN0_D21_OFFSET			0x0A80
#define TI814X_CONTROL_PADCONF_VIN0_D22_OFFSET			0x0A84
#define TI814X_CONTROL_PADCONF_VIN0_D23_OFFSET			0x0A88
#define TI814X_CONTROL_PADCONF_VIN0_DE0_MUX1_OFFSET		0x0A8C
#define TI814X_CONTROL_PADCONF_VIN0_DE1_OFFSET			0x0A90
#define TI814X_CONTROL_PADCONF_VIN0_FLD0_MUX1_OFFSET		0x0A94
#define TI814X_CONTROL_PADCONF_VIN0_FLD1_OFFSET			0x0A98
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC1_OFFSET		0x0A9C
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC0_OFFSET		0x0AA0
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR1_OFFSET		0x0AA4
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR0_OFFSET		0x0AA8
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C1_OFFSET		0x0AAC
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C0_OFFSET		0x0AB0
#define TI814X_CONTROL_PADCONF_VOUT1_FID_OFFSET			0x0AB4
#define TI814X_CONTROL_PADCONF_VOUT0_FID_OFFSET			0x0AB8
#define TI814X_CONTROL_PADCONF_VOUT0_CLK_OFFSET			0x0ABC
#define TI814X_CONTROL_PADCONF_VOUT0_HSYNC_OFFSET		0x0AC0
#define TI814X_CONTROL_PADCONF_VOUT0_VSYNC_OFFSET		0x0AC4
#define TI814X_CONTROL_PADCONF_VOUT0_AVID_OFFSET		0x0AC8
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C2_OFFSET		0x0ACC
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C3_OFFSET		0x0AD0
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C4_OFFSET		0x0AD4
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C5_OFFSET		0x0AD8
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C6_OFFSET		0x0ADC
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C7_OFFSET		0x0AE0
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C8_OFFSET		0x0AE4
#define TI814X_CONTROL_PADCONF_VOUT0_B_CB_C9_OFFSET		0x0AE8
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC2_OFFSET		0x0AEC
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC3_OFFSET		0x0AF0
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC4_OFFSET		0x0AF4
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC5_OFFSET		0x0AF8
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC6_OFFSET		0x0AFC
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC7_OFFSET		0x0B00
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC8_OFFSET		0x0B04
#define TI814X_CONTROL_PADCONF_VOUT0_G_Y_YC9_OFFSET		0x0B08
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR2_OFFSET		0x0B0C
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR3_OFFSET		0x0B10
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR4_OFFSET		0x0B14
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR5_OFFSET		0x0B18
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR6_OFFSET		0x0B1C
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR7_OFFSET		0x0B20
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR8_OFFSET		0x0B24
#define TI814X_CONTROL_PADCONF_VOUT0_R_CR9_OFFSET		0x0B28
#define TI814X_CONTROL_PADCONF_VOUT1_CLK_OFFSET			0x0B2C
#define TI814X_CONTROL_PADCONF_VOUT1_HSYNC_OFFSET		0x0B30
#define TI814X_CONTROL_PADCONF_VOUT1_VSYNC_OFFSET		0x0B34
#define TI814X_CONTROL_PADCONF_VOUT1_AVID_OFFSET		0x0B38
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C3_OFFSET		0x0B3C
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C4_OFFSET		0x0B40
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C5_OFFSET		0x0B44
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C6_OFFSET		0x0B48
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C7_OFFSET		0x0B4C
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C8_OFFSET		0x0B50
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C9_OFFSET		0x0B54
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC3_OFFSET		0x0B58
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC4_OFFSET		0x0B5C
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC5_OFFSET		0x0B60
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC6_OFFSET		0x0B64
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC7_OFFSET		0x0B68
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC8_OFFSET		0x0B6C
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC9_OFFSET		0x0B70
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR4_OFFSET		0x0B74
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR5_OFFSET		0x0B78
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR6_OFFSET		0x0B7C
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR7_OFFSET		0x0B80
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR8_OFFSET		0x0B84
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR9_OFFSET		0x0B88
#define TI814X_CONTROL_PADCONF_VOUT1_G_Y_YC2_OFFSET		0x0B8C
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR3_OFFSET		0x0B90
#define TI814X_CONTROL_PADCONF_VOUT1_R_CR2_OFFSET		0x0B94
#define TI814X_CONTROL_PADCONF_VOUT1_B_CB_C2_OFFSET		0x0B98
#define TI814X_CONTROL_PADCONF_RMII_REFCLK_OFFSET		0x0B9C
#define TI814X_CONTROL_PADCONF_MDIO_MCLK_OFFSET			0x0BA0
#define TI814X_CONTROL_PADCONF_MDIO_D_OFFSET			0x0BA4
#define TI814X_CONTROL_PADCONF_GMII0_TXCLK_OFFSET		0x0BA8
#define TI814X_CONTROL_PADCONF_GMII0_COL_OFFSET			0x0BAC
#define TI814X_CONTROL_PADCONF_GMII0_CRS_OFFSET			0x0BB0
#define TI814X_CONTROL_PADCONF_GMII0_RXER_OFFSET		0x0BB4
#define TI814X_CONTROL_PADCONF_GMII0_RXCLK_OFFSET		0x0BB8
#define TI814X_CONTROL_PADCONF_GMII0_RXD0_OFFSET		0x0BBC
#define TI814X_CONTROL_PADCONF_GMII0_RXD1_OFFSET		0x0BC0
#define TI814X_CONTROL_PADCONF_GMII0_RXD2_OFFSET		0x0BC4
#define TI814X_CONTROL_PADCONF_GMII0_RXD3_OFFSET		0x0BC8
#define TI814X_CONTROL_PADCONF_GMII0_RXD4_OFFSET		0x0BCC
#define TI814X_CONTROL_PADCONF_GMII0_RXD5_OFFSET		0x0BD0
#define TI814X_CONTROL_PADCONF_GMII0_RXD6_OFFSET		0x0BD4
#define TI814X_CONTROL_PADCONF_GMII0_RXD7_OFFSET		0x0BD8
#define TI814X_CONTROL_PADCONF_GMII0_RXDV_OFFSET		0x0BDC
#define TI814X_CONTROL_PADCONF_GMII0_GTXCLK_OFFSET		0x0BE0
#define TI814X_CONTROL_PADCONF_GMII0_TXD0_OFFSET		0x0BE4
#define TI814X_CONTROL_PADCONF_GMII0_TXD1_OFFSET		0x0BE8
#define TI814X_CONTROL_PADCONF_GMII0_TXD2_OFFSET		0x0BEC
#define TI814X_CONTROL_PADCONF_GMII0_TXD3_OFFSET		0x0BF0
#define TI814X_CONTROL_PADCONF_GMII0_TXD4_OFFSET		0x0BF4
#define TI814X_CONTROL_PADCONF_GMII0_TXD5_OFFSET		0x0BF8
#define TI814X_CONTROL_PADCONF_GMII0_TXD6_OFFSET		0x0BFC
#define TI814X_CONTROL_PADCONF_GMII0_TXD7_OFFSET		0x0C00
#define TI814X_CONTROL_PADCONF_GMII0_TXEN_OFFSET		0x0C04
#define TI814X_CONTROL_PADCONF_CLKIN32_OFFSET			0x0C08
#define TI814X_CONTROL_PADCONF_RESETN_OFFSET			0x0C0C
#define TI814X_CONTROL_PADCONF_NMIN_OFFSET			0x0C10
#define TI814X_CONTROL_PADCONF_RSTOUTN_OFFSET			0x0C14
#define TI814X_CONTROL_PADCONF_I2C0_SCL_OFFSET			0x0C18
#define TI814X_CONTROL_PADCONF_I2C0_SDA_OFFSET			0x0C1C
#define TI814X_CONTROL_PADCONF_USB_DRVVBUS_OFFSET		0x0C34

#define TI814X_CONTROL_PADCONF_MUX_SIZE				\
		(TI814X_CONTROL_PADCONF_USB_DRVVBUS_OFFSET + 0x4)

#endif
