/**	@file spi_flash_io.c
 *	@note HangZhou Hikvision System Technology Co., Ltd. All Right Reserved.
 *	@brief ʵspi flashдӿڲģ
 *
 *	@author		zhaoweibo
 *	@date		2011/04/10
 *
 *
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/time.h>
#include <linux/sched.h>
#include <linux/slab.h>


#include "hik_spi_flash_m25p16.h"
#include "hik_spi_flash_io.h"
#include "spi.h"
#include "spi_flash.h"
#include "spi_flash_internal.h"

//#define SPI_FLASH_DEBUG
#ifdef SPI_FLASH_DEBUG
#define DB(x) x
#else
#define DB(x)
#endif 

#define SPI_FLASH_BUS	0
#define SPI_FLASH_CS	0
#define SPI_FLASH_HZ	24000000	//24M Hz
#define SPI_FLASH_MODE		SPI_MODE_3

//#define ARRAY_SIZE(a)                    ((sizeof(a)) / (sizeof(a[0])))

#define INVALID_DEVICE_NUMBER	0xFFFFFFFF

SPI_FLASH_INFO* spi_flash_info = NULL;

//definitions
static struct mtd_info *spi_flash_probe(struct map_info *map);
static void spi_flash_destroy(struct mtd_info *mtd);
static int spi_flash_read(struct mtd_info *mtd, loff_t from, size_t len, size_t *retlen, u_char *buf);
static int spi_flash_write(struct mtd_info *mtd, loff_t from, size_t len, size_t *retlen, const u_char *buf);
static int spi_flash_erase(struct mtd_info *mtd, struct erase_info *instr);
static void spi_flash_sync(struct mtd_info *mtd);
static int spi_flash_suspend(struct mtd_info *mtd);
static void spi_flash_resume(struct mtd_info *mtd);
static int spi_flash_lock (struct mtd_info *mtd, loff_t ofs, uint64_t len);
static int spi_flash_unlock (struct mtd_info *mtd, loff_t ofs, uint64_t len);
static int spi_flash_block_isbad (struct mtd_info *mtd, loff_t ofs);
static int spi_flash_block_markbad (struct mtd_info *mtd, loff_t ofs);


static struct mtd_chip_driver spi_flash_chipdrv = 
{
    .probe	= spi_flash_probe,
    .destroy  = spi_flash_destroy,
    .name	= "spi_flash",
    .module	= THIS_MODULE
};


/*@fn		spi_flash_probe  
 * @brief		fill the structure of mtf_info  
 * @param[in]  
 *	map		--	the mapping infomation of spi flash
 * @param[out] 
 *	mtd_info	--	the pointer to mtd_info
 * @return	  
 */
static struct mtd_info *spi_flash_probe(struct map_info *map)
{
    struct mtd_info *mtd = NULL;
    SPI_FLASH_INFO *flash_info = NULL;
    
    /* allocate the memory for the mtd_info */
    mtd = kmalloc(sizeof(*mtd), GFP_KERNEL);
    if(!mtd)
    {
        printk(KERN_NOTICE "\nERROR: %s - Failed to allocate memory for mtd structure",__FUNCTION__);
        return NULL;
    }
    
    /* allocate memory for the spi flash private structure */
    flash_info = kmalloc(sizeof(SPI_FLASH_INFO), GFP_KERNEL);
    if(!flash_info) 
    {
        printk(KERN_NOTICE "\nERROR: %s - Failed to allocate memory for spi_flash structure",__FUNCTION__);
        kfree(mtd);
        return NULL;
    }
    	
    /* clear both structures before usage */
    memset(mtd, 0, sizeof(*mtd));
    memset(flash_info, 0, sizeof(*flash_info));
        
    /* Try to detect the flash and initialize it over SPI */	
    flash_info->baseAddr       = (UINT32)map->virt;	//зʱṩ߼ַ
    	flash_info->index            = INVALID_DEVICE_NUMBER; /* will be detected in init */	
    
    DB(printk("INFO: %s - Base address 0x%08x\n", __FUNCTION__, map->virt));
    
    if (ERR_OK != spi_flash_init(flash_info))
    {
        printk(KERN_NOTICE "ERROR: %s - Failed to initialize the spi_flash.\n", __FUNCTION__);
        kfree(mtd);
        kfree(flash_info);
        return NULL;
    }
    
    //ṩһⲿʽӿ
    spi_flash_info =  flash_info;
    
    /* After success fill in the MTD structure with the appropriate info */
    mtd->erasesize = flash_info->sectorSize;
    mtd->size = flash_info->sectorSize * flash_info->sectorNumber;
    mtd->priv = map; /*spi_flash;*/
    mtd->type = MTD_NORFLASH;
    mtd->erase = spi_flash_erase;
    mtd->read = spi_flash_read;
    mtd->write = spi_flash_write;
    mtd->sync = spi_flash_sync;
    mtd->suspend = spi_flash_suspend;
    mtd->resume = spi_flash_resume;	
    mtd->lock = spi_flash_lock;
    mtd->unlock = spi_flash_unlock;
    mtd->block_isbad = spi_flash_block_isbad;
    mtd->block_markbad = spi_flash_block_markbad;	
    mtd->flags = (MTD_WRITEABLE | MTD_BIT_WRITEABLE); /* just like MTD_CAP_NORFLASH */
    mtd->name = map->name;
    mtd->writesize = 1;
    
    map->fldrv = &spi_flash_chipdrv;
    map->fldrv_priv = flash_info;
    
    /* Print some debug messages with the detected spi flash info */
    DB(printk("\nINFO: %s - Detected SPI Flash device (size %d)", __FUNCTION__, mtd->size));
    DB(printk("\n           Base Address      : 0x%08x", flash_info->baseAddr));
    DB(printk("\n           Manufacturer ID  : 0x%02x", flash_info->manufacturerId));
    DB(printk("\n           Device ID           : 0x%04x", flash_info->deviceId));
    DB(printk("\n           Sector Size        : 0x%x", flash_info->sectorSize));
    DB(printk("\n           Sector Number    : %d", flash_info->sectorNumber));
    DB(printk("\n           Page Size           : %d", flash_info->pageSize));
    
    printk("SPI Serial flash detected @ 0x%08x, %dKB (%dsec x %dKB)\n",
             flash_info->baseAddr, ((flash_info->sectorNumber * flash_info->sectorSize)/1024), 
             flash_info->sectorNumber, (flash_info->sectorSize/1024));
    
    __module_get(THIS_MODULE);
    return mtd;
}

static void spi_flash_destroy(struct mtd_info *mtd)
{
    struct map_info *map = mtd->priv;
    SPI_FLASH_INFO *flash_info = map->fldrv_priv;
    
    DB(printk("\nINFO: %s called", __FUNCTION__));
    
    /* free memory allocated at probe for the private spi_flash structure */
    if (flash_info->flash)
        kfree(flash_info->flash);
    if (flash_info)
        kfree(flash_info);
    if(mtd)
        kfree(mtd);
}
/*@fn		spi_flash_init  
 * @brief		init spi flash  
 * @param[in]  
 *	flash_info		--	infomation of spi flash
 * @param[out] 
 *	RET_STATUS	--	result
 * @return	  
 */
RET_STATUS spi_flash_init (SPI_FLASH_INFO * flash_info)
{
    RET_STATUS ret;
    UINT8 idcode[5];
    struct spi_slave* spi = NULL;
    struct spi_flash *flash = NULL;

    /* check for NULL pointer */
    if (NULL == flash_info)
    {
        printk("%s ERROR: Null pointer parameter!\n", __FUNCTION__);
        return ERR_BAD_PARAM;
    }

   //set up flash
   spi = spi_setup_slave(SPI_FLASH_BUS, SPI_FLASH_CS, 
   							SPI_FLASH_HZ, SPI_FLASH_MODE, flash_info->baseAddr);

    ret = spi_claim_bus(spi);
    if (ret) 
    {
		printk("SF: Failed to claim SPI bus: %d\n", ret);
		goto err_claim_bus;
    }

    /* Read the ID codes */
    ret = spi_flash_cmd(spi, CMD_READ_ID, &idcode, sizeof(idcode));
    if (ret)
	goto err_read_id;


    DB(printf("SF: Got idcode %02x %02x %02x %02x %02x\n", idcode[0],
			idcode[1], idcode[2], idcode[3], idcode[4]));

    switch(idcode[0])
    {
#ifdef CONFIG_SPI_FLASH_SPANSION
    case 0x01:
        flash = spi_flash_probe_spansion(spi, idcode);
        break;
#endif
#ifdef CONFIG_SPI_FLASH_ATMEL
    case 0x1F:
        flash = spi_flash_probe_atmel(spi, idcode);
        break;
#endif
#ifdef CONFIG_SPI_FLASH_MACRONIX
    case 0xc2:
        flash = spi_flash_probe_macronix(spi, idcode);
        break;
#endif
#ifdef CONFIG_SPI_FLASH_WINBOND
    case 0xef:
        flash = spi_flash_probe_winbond(spi, idcode);
        break;
#endif
#ifdef CONFIG_SPI_FLASH_STMICRO
    case 0x20:
        flash = spi_flash_probe_stmicro(spi, idcode);
        break;
#endif
#ifdef CONFIG_SPI_FLASH_SST
    case 0xBF:
        flash = spi_flash_probe_sst(spi, idcode);
        break;
#endif
    default:
        printk("SF: Unsupported manufacturer %02X\n", idcode[0]);
        flash = NULL;
        break;
    }

    spi_release_bus(spi);
	
    if(!flash)
    {
        printk("%s ERROR: Unknown SPI flash device!\n", __FUNCTION__);
        printk("ID code 0x%02X 0x%02X 0x%02X 0x%02X 0x%02X \n", 
	  		idcode[0], idcode[1], idcode[2], idcode[3], idcode[4]);
        return ERR_FAIL;
    }

    /* fill the info based on the model detected */
    flash_info->spi = spi;
    flash_info->flash = flash;
    flash_info->sectorSize = flash->sectorSize;   
    flash_info->sectorNumber = flash->sectorNumber;
    flash_info->pageSize = flash->pageSize;

   return ERR_OK;
	
err_read_id:
    spi_release_bus(spi);
err_claim_bus:
    spi_free_slave(spi);
    return ERR_FAIL;
}

/*@fn		  spi_flash_read
 * @brief		  read section from spi flash
 * @param[in]  
 *	mtd		--	pointer to the mtd_info which includes operation to flash
 *	from		--	address of flash to read from
 *	len		--	length of read
 *	retlen	--	length of read actually
 *	buf		--	buffer of storing data read from flash
 * @param[out] 
 *	ERR_OK	--	succeed
 *	other	--	fail
 * @return	  
 */
static int spi_flash_read(struct mtd_info *mtd, loff_t from, size_t len,
	size_t *retlen, u_char *buf)
{
    struct map_info *map = (struct map_info *)(mtd->priv);
    SPI_FLASH_INFO *flash_info = (SPI_FLASH_INFO *)map->fldrv_priv;
    UINT32 offset = ((UINT32)from);
    INT ret;
    
    if(NULL == mtd)
    {
        printk(KERN_NOTICE"\nERROR:%s -Bad param 'mtd'", __FUNCTION__);
        return ERR_BAD_PARAM;
    }
    
    *retlen = 0;
    
#ifdef SPI_FLASH_DEBUG
    struct timeval start, end; 
#endif	
    
    DB(printk("\nINFO:%s  -addr %X len %d",__FUNCTION__, offset, (int)len));
    
    if(NULL == buf || len <= 0)
    {
        printk(KERN_NOTICE"\nERROR:%s -Bad param 'buf' or 'flash_info'", __FUNCTION__);
        return ERR_BAD_PARAM;
    }
    
#ifdef SPI_FLASH_DEBUG	
    do_gettimeofday(&start);
#endif
    
    if ((ret = flash_read(flash_info->flash, offset, len, buf)) != ERR_OK)
    {
        printk(KERN_NOTICE "\nERROR: %s - Failed to read block. ret: %d", __FUNCTION__, ret);
        *retlen = 0;
        return ret;
    }
#ifdef SPI_FLASH_DEBUG
    do_gettimeofday(&end);
    //DB(printk("v = %.1f ", len/10/(start - end)));
    DB(printk("Rd %d  T %d ms ",
    			len, ((end.tv_sec - start.tv_sec) *1000 + (end.tv_usec - start.tv_usec) / 1000)));
    DB(printk("- OK"));
#endif
    *retlen = len;
    return ERR_OK;	
}

UINT32 spi_flash_get_size(SPI_FLASH_INFO* flash_info)
{
    /* check for NULL pointer */
    if (NULL == flash_info)
    {
        printk("%s ERROR: Null pointer parameter!\n", __FUNCTION__);
        return ERR_BAD_PARAM;
    }

    return (flash_info->flash->size);
}

/*@fn		  spi_flash_write
 * @brief		  write data to a section of spi flash
 * @param[in]  
 *	mtd		--	pointer to the mtd_info which includes operation to flash
 *	to		--	address of flash to write to
 *	len		--	length of write
 *	retlen	--	length of written actually
 *	buf		--	buffer of storing data written to flash
 * @param[out] 
 *	ERR_OK	--	succeed
 *	other	--	fail
 * @return	  
 */
static int spi_flash_write(struct mtd_info *mtd, loff_t to, size_t len,
	size_t *retlen, const u_char *buf)
{
    struct map_info *map = mtd->priv;
    SPI_FLASH_INFO *flash_info = map->fldrv_priv;
    UINT32 offset = ((UINT32)to);
    INT ret = 0;
    
#ifdef SPI_FLASH_DEBUG
    struct timeval start, end; 
#endif	
    *retlen = 0;
    
    DB(printk("\nINFO: %s - addr 0x%X, len %d",__FUNCTION__, offset, len));
    
#ifdef SPI_FLASH_DEBUG
    do_gettimeofday(&start);	
#endif
    
    if((NULL == buf) || len <= 0)
    {
        printk(KERN_NOTICE"\nERROR:%s -Bad param 'buf' or 'flash_info'", __FUNCTION__);
        return ERR_BAD_PARAM;
    }
    
     if ((offset + len) > spi_flash_get_size(flash_info))
     {
        printk("%s WARNING: Write exceeds flash size!WR addr: 0x%X size:0x%X FlashSize: 0x%X\n",
    			__FUNCTION__, offset, len, spi_flash_get_size(flash_info));
        return -ENOMEM;
      }
    
    ret = flash_write(flash_info->flash, offset, len, (void*)buf);
    if (ERR_OK != ret)
    {
        printk(KERN_NOTICE "\nERROR: %s - Failed to write block!len:%d err_ret %d",
    				__FUNCTION__, len, ret);
        *retlen = -1;
        return ret;
    }
    
    *retlen = len;
#ifdef SPI_FLASH_DEBUG
    do_gettimeofday(&end);
    DB(printk(" v = %.1f bytes/ms - OK", len /((end.tv_sec - start.tv_sec) *1000.0 + (end.tv_usec - start.tv_usec) / 1000.0) ));
    DB(printk("- OK"));
#endif
    
    return ERR_OK;	
}

/*@fn		  spi_flash_erase
 * @brief		  erase a section of spi flash
 * @param[in]  
 *	mtd		--	pointer to the mtd_info which includes operation to flash
 *	instr		--	erase infomation 
 * @param[out] 
 *	ERR_OK	--	succeed
 *	other	--	fail
 * @return	  
 */
static int spi_flash_erase(struct mtd_info *mtd, struct erase_info *instr)
{
    struct map_info *map = mtd->priv;
    SPI_FLASH_INFO *flash_info = map->fldrv_priv;
    
    DB(printk("\nINFO: %s - addr %08x, len %d",__FUNCTION__, instr->addr, instr->len));
    
    if(instr->addr & (mtd->erasesize - 1))
    {
        printk(KERN_NOTICE "\nError: %s - Erase address not sector alligned",__FUNCTION__);
        return -EINVAL;
    }
    if(instr->len & (mtd->erasesize - 1))
    {
        printk(KERN_NOTICE "\nError: %s - Erase length is not sector alligned",__FUNCTION__);
        return -EINVAL;
    }
    if((instr->len + instr->addr) > mtd->size)
    {
        printk(KERN_NOTICE "\nError: %s - Erase exceeded flash size",__FUNCTION__);
        return -ENOMEM;
    }
    
    if (ERR_OK != flash_erase(flash_info->flash, instr->addr, instr->len))
    {
        printk(KERN_NOTICE "\nERROR: %s - erase addr: 0x%llX len: %lld sectSize %d k \n",
    		 __FUNCTION__, instr->addr, instr->len, flash_info->sectorSize >> 10);
        instr->state = MTD_ERASE_FAILED;
        return -1;
    }
    
    DB(printk(" - OK"));
    
    instr->state = MTD_ERASE_DONE;
    mtd_erase_callback(instr);
    
    return ERR_OK;
}

static int spi_flash_lock (struct mtd_info *mtd, loff_t ofs, uint64_t len)
{
    printk("\nNotice: Serial SPI flash (%s) lock per sector is not supported!\n", mtd->name);
    return 0;
}

static int spi_flash_unlock (struct mtd_info *mtd, loff_t ofs, uint64_t len)
{
    printk("\nNotice: Serial SPI flash (%s) unlock per sector is not supported!\n", mtd->name);
    return 0;
}

static void spi_flash_sync(struct mtd_info *mtd)
{
    DB(printk("\nINFO: %s called - DUMMY", __FUNCTION__));
}

static int spi_flash_suspend(struct mtd_info *mtd)
{
    DB(printk("\nINFO: %s called - DUMMY()", __FUNCTION__));
    return 0;
}

static void spi_flash_resume(struct mtd_info *mtd)
{
    DB(printk("\nINFO: %s called - DUMMY", __FUNCTION__));
}

static int spi_flash_block_isbad (struct mtd_info *mtd, loff_t ofs)
{
    return 0;
}

static int spi_flash_block_markbad (struct mtd_info *mtd, loff_t ofs)
{
    DB(printk("\nINFO: %s called - DUMMY", __FUNCTION__));
    return 0;
}


static int __init spi_flash_io_init(void)
{
    DB(printk("\nINFO: %s - MTD spi Flash chip driver.", __FUNCTION__));
    register_mtd_chip_driver(&spi_flash_chipdrv);
    return 0;
}

static void __exit spi_flash_io_exit(void)
{
    DB(printk(KERN_ALERT "\nINFO: %s - MTD spi Flash driver exit", __FUNCTION__));
    unregister_mtd_chip_driver(&spi_flash_chipdrv);
}

module_init(spi_flash_io_init);
module_exit(spi_flash_io_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Zhao Weibo Authried <zhaoweibo@hikvision.com>");
MODULE_DESCRIPTION("Probe code for spi flash chips");
