/**	@file hik_spi_flash_m25p16.h
 *	@note HangZhou Hikvision System Technology Co., Ltd. All Right Reserved.
 *	@brief ʵM25P16 flash
 *
*	@author		zhaoweibo
 *	@date		2011/03/25
 *
*
 */

#ifndef __HIK_SPI_FLASH_M25P16_H__
#define __HIK_SPI_FLASH_M25P16_H__

#define M25P16_BASE_ADDR  (0x48030100)

#define M25P16_MANUFACTURE_ID		0x20

/////spi flash instruction set//////////////////////////
#define SF_WREN				0x06	//write enable
#define SF_WRDI				0x04	//write disable

//register 
#define SF_RDSR				0x05	//read status register
#define SF_WRSR				0x01	//write status register

#define SF_READ				0x03
#define SF_FAST_READ		0x0B
#define SF_PP				0x02	//page program

#define SF_SE				0xD8	//64kb
#define SF_BE				0xC7	//the entire memory

#define SF_M25P16_PWR_DOWN		0xB9
#define SF_M25P16_RLE_PWR_DOWN	0xAB	//release power down

#define SF_RDID				0x9F	//Jedec ID (Manufacture ID and Device ID)

#define SF_M25P16_MAX_SIZE			(0x200000)
#define SF_M25P16_SECT_SIZE		(64 * 1024)
#define SF_M25P16_PAGE_SIZE		256
#define SF_M25P16_SECT_NUM		32

#define INVALID_CMD			0x00	//flash don't support the cmd

#define M25P16_DEVICE_ID	0x15

#define M25P16_MAX_SPI_FREQ	50000000		//50M Hz

//////flash status///////////////////////
#define SF_STATUS_BUSY		0x01	//flash is in busy  ex. erase program or write status register

/////////flash status mask///////////////////////////
#define FLAG_BIT_WIP	(0x01)		//write in progress			1 - busy	0 - no write cycle
#define FLAG_BIT_WEL	(0x01 << 1)	//write enable latch bit
#define FLAG_BIT_SRWD	(0x01 << 7)	//status register write protected
#define WP_M25P16_MASK	(0x0F << 2)	//memory protection

#endif /*__HIK_SPI_FLASH_M25P16_H__*/

